/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: btctl-obex.c,v 1.5 2002/08/29 14:14:29 kds Exp $

   btctl - driver control program

   Fixes:	Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
                Imre Deak <ext-imre.deak@nokia.com>
*/

#include <affix/config.h>

#include <glib.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <sys/socket.h>
#include <sys/time.h>
#include <sys/stat.h>	//XXX
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>

#include <stdio.h>
#include <unistd.h>
#include <errno.h>

#include <openobex/obex.h>

#include <affix/bluetooth.h>
#include <affix/btcore.h>
#include <affix/debug.h>
#include <affix/obex.h>

#include "btctl.h"

//char	*obex_root_path="/var/spool/affix";

union {
	struct sockaddr_affix	bt;
	struct sockaddr_in	in;
	char			port[IFNAMSIZ];
} saddr;

int print_speed(char *format, char *name, struct timeval *tv_start)
{
	struct timeval	tv_end;
	long int	sec, rsec;
	long int	usec, rusec;
	long int	size;
	double		speed;

	size = get_filesize(name);
	gettimeofday(&tv_end, NULL);
	sec = tv_end.tv_sec - tv_start->tv_sec;
	usec = (1000000 * sec) + tv_end.tv_usec - tv_start->tv_usec;
	rsec = usec/1000000;
	rusec = (usec - (rsec * 1000000))/10000;
	speed = (double)(size)/((double)(rsec) + (double)(rusec)/100);

	printf(format, size, rsec, rusec, speed);

	return 0;
}

int print_obex_error(int err)
{
	if (err) {
		if (err < 0)
			perror("System error");
		else
			printf("OBEX error (%#02x): %s\n", err, obex_error(err));
	}
	return 0;
}

static void set_perm(char *perm, char *str)
{
	int	c;
	
	//printf("In set_perm\n");
	for (;*str; str++) {
		c = tolower(*str);
		//printf("char: %c\n", c);
		switch (c) {
			case 'r':
				perm[1] = 'r';
				break;
			case 'w':
				perm[2] = 'w';
				break;
			case 'd':
				perm[3] = 'd';
			case 'x':
				perm [4] = 'x';
				break;
		}
	}
}

#define PERM_SIZE	6
void print_folder(const char *buf)
{
	char	*next = (char*)buf, *elem, *attrs, *attr, *value;
	char	*size = NULL, *name = NULL;
	int	count = 0;
	char	str[80], perm[PERM_SIZE];


	while ((elem = xml_element(&next, &attrs))) {
		if (strcmp(elem, "folder-listing") == 0)
			break;
	}

	while ((elem = xml_element(&next, &attrs))) {
		if (strcmp(elem, "/folder-listing") == 0)
			break;
		size = NULL, name = NULL;
		count = 0;
		//printf("element: %s\n", elem);
		//printf("attr left: %s\n", attrs);
		memset(perm, '-', PERM_SIZE);
		perm[PERM_SIZE-1] = '\0';
		if (strcmp(elem, "folder") == 0)
			perm[0] = 'd';
		else if (strcmp(elem, "file") == 0)
			;
		else if (strcmp(elem, "parent-folder") == 0) {
			perm[0] = 'd';
			name = "..";
		} else {
		}

		// get attributes
		while ((attr = xml_attribute(&attrs, &value))) {
			//printf("attr: %s, value: %s\n", attr, value);
			if (strcmp(attr, "user-perm") == 0)
				set_perm(perm, value);
			else if ( name == NULL && strcmp(attr, "name") == 0 )
				name = value;
			else if ( size == NULL && strcmp(attr, "size") == 0 )
				size = value;
		}
		count += sprintf(str+count, "%s\t\t%s\t\t%s", perm,
				size?size:"0", name?name:"<no name>");
		printf("%s\n", str);
	}
}

int obex_setaddress(void)
{
	int	err;
	
	if (linkmode == LINK_BLUETOOTH) {
		if (sdpmode) {
			if (argv[argind] == NULL)
				return -1;
		} else {
			if (argv[argind] == NULL || argv[argind+1] == NULL)
				return -1;
		}

		err = get_bda(&saddr.bt.bda, argv[argind]);
		if (err) {
			return -1;
		}
		if (sdpmode) { 
			saddr.bt.port = 0;	// SDP 
			argind += 1;
		} else {
			saddr.bt.port = atoi(argv[argind+1]);
			argind += 2;
		}

	} else if (linkmode == LINK_TCP) {
		if (argv[argind] == NULL) {
			return -1;
			err = inet_aton("127.0.0.1", &saddr.in.sin_addr);
		} else {
			err =  inet_aton(argv[argind], &saddr.in.sin_addr);
			if (err == 0) {
				struct hostent	*he;
				he = gethostbyname(argv[argind]);
				if (he == NULL)
					return -1;
				saddr.in.sin_addr.s_addr = *(uint32_t*)he->h_addr;
			}
			argind++;
		}
	} else {
		if (argv[argind] == NULL) {
			return -1;
		}
		strcpy(saddr.port, argv[argind]);
		argind++;
	}
	
	return 0;
}


static obexclt_t	*handle = NULL;

int _cmd_open(struct btctl_command *cmd)
{
	int	err;

	err = obex_setaddress();
	if (err) {
		printf("Address error\n");
		return -1;
	}

	handle = obex_connect(&saddr, &err);
	if (handle == NULL) {
		printf("Unable to connect.\n");
		print_obex_error(err);
		return err;
	}
	printf("Connected.\n");
	return 0;
}

int _cmd_close(struct btctl_command *cmd)
{
	if (handle) {
		obex_disconnect(handle);
		handle = NULL;
	}

	return 0;
}

int _cmd_ls(struct btctl_command *cmd)
{
	int	err;
	char	*buf;

	if (!handle) {
		printf("Not connected.\n");
		return -1;
	}
	buf = malloc(0xFFFF);
	err = obex_browse(handle, NULL, buf);
	if (err) {
		printf("Browsing error.\n");
		print_obex_error(err);
	} else {
		//printf("%s\n", buf);
		print_folder(buf);
		printf("Command complete.\n");
	}
	free(buf);
	return 0;
}

int _cmd_put(struct btctl_command *cmd)
{
	int		err;
	char		*name;
	struct timeval	tv_start;

	gettimeofday(&tv_start, NULL);

	if (!handle) {
		printf("Not connected.\n");
		return -1;
	}
	if (argv[argind] == NULL) {
		printf("No file name.\n");
		return -1;
	}
	name = argv[argind];

	err = obex_put(handle, name);
	if (err) {
		printf("File transfer error.\n");
		print_obex_error(err);
	} else {
		printf("Transfer complete.\n");
		print_speed("%ld bytes sent in %ld.%ld secs (%.2f B/s)\n", name, &tv_start);
	}		

	return 0;
}

int _cmd_get(struct btctl_command *cmd)
{
	int		err;
	char		*name;
	struct timeval	tv_start;

	gettimeofday(&tv_start, NULL);

	if (!handle) {
		printf("Not connected.\n");
		return -1;
	}
	if (argv[argind] == NULL) {
		printf("No file name.\n");
		return -1;
	}
	name = argv[argind];

	err = obex_get(handle, name);
	if (err) {
		printf("File transfer error.\n");
		print_obex_error(err);
	} else {
		printf("Transfer complete.\n");
		print_speed("%ld bytes received in %ld.%ld secs (%.2f B/s)\n", name, &tv_start);
	}

	return 0;
}

int _cmd_push(struct btctl_command *cmd)
{
	int		err;
	char		*name;
	struct timeval	tv_start;

	gettimeofday(&tv_start, NULL);


	err = obex_setaddress();
	if (err) {
		printf("Address error\n");
		return -1;
	}

	if (argv[argind] == NULL) {
		printf("No file name.\n");
		return -1;
	}
	name = argv[argind];

	err = obex_push(&saddr, name);
	if (err) {
		printf("Object pushing error.\n");
		print_obex_error(err);
	} else {
		printf("Transfer complete.\n");
		print_speed("%ld bytes sent in %ld.%ld secs (%.2f B/s)\n", name, &tv_start);
	}

	return 0;
}

int _cmd_rm(struct btctl_command *cmd)
{
	int	err;
	char	*name;

	if (!handle) {
		printf("Not connected.\n");
		return -1;
	}
	if (argv[argind] == NULL) {
		printf("No file name.\n");
		return -1;
	}
	name = argv[argind];

	err = obex_remove(handle, name);
	if (err) {
		printf("File removal error.\n");
		print_obex_error(err);
	} else
		printf("Command complete.\n");

	return 0;
}

int _cmd_cd(struct btctl_command *cmd)
{
	int	err;
	char	*name;

	if (!handle) {
		printf("Not connected.\n");
		return -1;
	}
	name = argv[argind];
	err = obex_setpath(handle, name);
	if (err) {
		printf("Command error.\n");
		print_obex_error(err);
	} else
		printf("Command complete.\n");

	return 0;
}

int _cmd_mkdir(struct btctl_command *cmd)
{
	int	err;
	char	*name;

	if (!handle) {
		printf("Not connected.\n");
		return 0;
	}
	name = argv[argind];
	err = obex_mkdir(handle, name);
	if (err) {
		printf("Command error.\n");
		print_obex_error(err);
	} else
		printf("Command complete.\n");

	return 0;
}


// primary commands
//
int cmd_open(struct btctl_command *cmd)
{
	int	err;

	if (!ftpmode) {
		printf("Command available only in FTP mode.\n");
		return -1;
	}

	err = _cmd_open(cmd);

	return err;
}

int cmd_close(struct btctl_command *cmd)
{
	int	err;

	if (!ftpmode) {
		printf("Command available only in FTP mode.\n");
		return -1;
	}

	err = _cmd_close(cmd);

	return err;
}
	
int cmd_ls(struct btctl_command *cmd)
{
	int	err;

	if (!ftpmode) {
		btlog_mask |= BTDEBUG_MODULE_OBEX;
		btdaemon = 1;

		err = obex_setaddress();
		if (err) {
			printf("Address error\n");
			return 1;
		}

		handle = obex_connect(&saddr, &err);
		if (handle == NULL) {
			printf("Unable to connect.\n");
			print_obex_error(err);
			return err;
		}
	}

	err = _cmd_ls(cmd);
	
	if (!ftpmode) {
		obex_disconnect(handle);
		btlog_mask &= ~BTDEBUG_MODULE_OBEX;
	}

	return err;
}

int cmd_put(struct btctl_command *cmd)
{
	int	err;

	if (!ftpmode) {
		btlog_mask |= BTDEBUG_MODULE_OBEX;
		btdaemon = 1;

		err = obex_setaddress();
		if (err) {
			printf("Address error\n");
			return 1;
		}

		if (argv[argind] == NULL) {
			printf("No file name.\n");
			return 1;
		}

		handle = obex_connect(&saddr, &err);
		if (handle == NULL) {
			printf("Unable to connect.\n");
			print_obex_error(err);
			return err;
		}
	}

	err = _cmd_put(cmd);

	if (!ftpmode) {
		obex_disconnect(handle);
		btlog_mask &= ~BTDEBUG_MODULE_OBEX;
	}

	return err;
}

int cmd_get(struct btctl_command *cmd)
{
	int	err;

	if (!ftpmode) {
		btlog_mask |= BTDEBUG_MODULE_OBEX;
		btdaemon = 1;

		err = obex_setaddress();
		if (err) {
			printf("Address error\n");
			return 1;
		}

		if (argv[argind] == NULL) {
			printf("No file name\n");
			return 1;
		}

		handle = obex_connect(&saddr, &err);
		if (handle == NULL) {
			printf("Unable to connect.\n");
			print_obex_error(err);
			return err;
		}
	}
	
	err = _cmd_get(cmd);

	if (!ftpmode) {
		obex_disconnect(handle);
		btlog_mask &= ~BTDEBUG_MODULE_OBEX;
	}

	return err;
}

int cmd_push(struct btctl_command *cmd)
{
	int	err;

	btlog_mask |= BTDEBUG_MODULE_OBEX;
	btdaemon = 1;

	err = _cmd_push(cmd);

	btlog_mask &= ~BTDEBUG_MODULE_OBEX;

	return err;
}

int cmd_rm(struct btctl_command *cmd)
{
	int	err;

	if (!ftpmode) {
		btlog_mask |= BTDEBUG_MODULE_OBEX;
		btdaemon = 1;

		err = obex_setaddress();
		if (err) {
			printf("Address error\n");
			return 1;
		}

		if (argv[argind] == NULL) {
			printf("No file name\n");
			return 1;
		}

		handle = obex_connect(&saddr, &err);
		if (handle == NULL) {
			printf("Unable to connect.\n");
			print_obex_error(err);
			return err;
		}
	}
	
	err = _cmd_rm(cmd);

	if (!ftpmode) {
		obex_disconnect(handle);
		btlog_mask &= ~BTDEBUG_MODULE_OBEX;
	}

	return err;
}

int cmd_cd(struct btctl_command *cmd)
{
	int	err;

	if (!ftpmode) {
		printf("Command available only in FTP mode.\n");
		return -1;
	}

	err = _cmd_cd(cmd);

	return err;
}

int cmd_mkdir(struct btctl_command *cmd)
{
	int	err;

	if (!ftpmode) {
		printf("Command available only in FTP mode.\n");
		return -1;
	}

	err = _cmd_mkdir(cmd);

	return err;
}


