/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author:Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   */

/*
   $Id: btctl-sdp.c,v 1.4 2002/07/19 15:18:34 kds Exp $

	Fixes:	Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>
		Manel Guerrero Zapata <manel.guerrero-zapata@nokia.com>

		Dmitry Kasatkin		: search added, fixes
*/

#include <affix/config.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>


#include <affix/sdp.h>
#include <affix/sdpclt.h>

#include "btctl.h"

BD_ADDR		bdAddr;
SDPServerHandle srvHandle;

char UUID_str[MAX_LEN_UUID_STR];

int _openConnection(char *target)
{
	int status;
	INQUIRY_ITEM inqItem[10];

	if (target == NULL) {
		int duration = 5;
		uint8_t count = 0;

		status = SDPGeneralInquiry(inqItem, 10, duration, &count);
		if (count != 0) {
			bdAddr = inqItem[0].bda;
		}
	} else {
		if (strcmp(target, "local") == 0) {
			printf("Opening local connection\n");
			bdAddr = SDPGetLocalBDAddr();
		} else {
			char bdStr[20];

			status = get_bda(&bdAddr, target);
			if (status) {
				printf("Incorrect address given\n");
				return 1;
			}
			bda2str(bdStr, &bdAddr);
			printf("Connecting to host %s ...\n", bdStr);
		}
	}
	
	status = SDPInit(0);
	if (status < 0)
		return status;

	srvHandle = SDPOpenServerConnection(bdAddr);
	if (srvHandle < 0)
		return -1;

	return 0;
}

void _closeConnection(void)
{
	if (srvHandle)
		SDPCloseServerConnection(srvHandle);
}

void GFuncPrintGroups(gpointer value, gpointer userData)
{
	UUID *pUUID= NULL;

	pUUID = (UUID *)value;
	UUID2strn(pUUID, UUID_str, MAX_LEN_UUID_STR);
	if (UUID2value(pUUID) == PublicBrowseGroup)
		printf("  \"PublicBrowseGroup\" (0x%s)\n", UUID_str);
	else
		printf("  0x%s\n", UUID_str);
}


void GFuncPrintServiceClass(gpointer value, gpointer userData)
{
	UUID *pUUID= NULL;
	char ServiceClassUUID_str[MAX_LEN_SERVICECLASS_UUID_STR];

	pUUID = (UUID *)value;
	UUID2strn(pUUID, UUID_str, MAX_LEN_UUID_STR);
	ServiceClassUUID2strn(pUUID, ServiceClassUUID_str, MAX_LEN_SERVICECLASS_UUID_STR);
	printf("  \"%s\" (0x%s)\n", ServiceClassUUID_str, UUID_str);
}


void GFuncPrintProtocolDescriptor(gpointer value, gpointer userData)
{
	ProtocolDescriptor *pPD = NULL;
	char ProtocolUUID_str[MAX_LEN_PROTOCOL_UUID_STR];

	pPD = (ProtocolDescriptor *)value;
	// printf("  Protocol Descriptors:\n");
	UUID2strn(&(pPD->protocolId), UUID_str, MAX_LEN_UUID_STR);
	ProtocolUUID2strn(&(pPD->protocolId), ProtocolUUID_str, MAX_LEN_PROTOCOL_UUID_STR);
	printf("  \"%s\" (0x%s)\n", ProtocolUUID_str, UUID_str);
	if (pPD->portNumber) {
		printf("    Port/Channel: %d\n", pPD->portNumber);
	}
	if (pPD->versionNumber) {
		printf("    Version: %d\n", pPD->versionNumber);
	}
}


void GFuncPrintAccessProtocols(gpointer value, gpointer userData)
{
	GSList *protDescSeq = NULL;

	protDescSeq = (GSList *)value;
	//printf("Access Protocols:\n");
	g_slist_foreach(protDescSeq, GFuncPrintProtocolDescriptor, NULL);
}


void GFuncPrintProfileDescriptor(gpointer value, gpointer userData)
{
	ProfileDescriptor *profDesc = NULL;
	char ProfileDescriptorUUID_str[MAX_LEN_PROFILEDESCRIPTOR_UUID_STR];

	profDesc = (ProfileDescriptor *)value;
	UUID2strn(&(profDesc->profileUUID), UUID_str, MAX_LEN_UUID_STR);
	ProfileDescriptorUUID2strn(&(profDesc->profileUUID), ProfileDescriptorUUID_str, MAX_LEN_PROFILEDESCRIPTOR_UUID_STR);
	printf("  \"%s\" (0x%s)\n", ProfileDescriptorUUID_str, UUID_str);
	if (profDesc->vnum) {
		printf("    Version: %d\n", profDesc->vnum);
	}
}

int extractGroups(UUID *groupToExtract)
{
	int status = -1, i;
	uint16_t count = 0;
	GSList *searchList = NULL;
	GSList *attrIdList = NULL;
	GSList *svcRecList = NULL;
	uint32_t range = 0x0000ffff;
	// uint32_t range = 0x00000004;
	// uint32_t range = 0x00000100;
	int groupIDStatus = -1;
	// UUID serviceClassUUID;
	GSList *pGSList = NULL;
	AccessProtocols *pAccessProtocols = NULL;

	searchList = g_slist_append(searchList, groupToExtract);
	attrIdList = g_slist_append(attrIdList, &range);
	status = __SDPServiceSearchAttributeRequest(srvHandle, searchList, 
			RangeOfAttributes, attrIdList, 65535, &svcRecList, &count);
	//printf("Status : %d Count : %d\n", status, count);

	if (status || count == 0)
		return status;

	for (i = 0; i < (int)g_slist_length(svcRecList); i++) {
		ServiceRecord *svcRec;
		printf("==============================\n");

		svcRec = (ServiceRecord *)g_slist_nth_data(svcRecList, i);
		printSDPDataAsServiceRecord(svcRec);
		printf("------------------------------\n");

		printf("SvcRecHdl: 0x%x\n", svcRec->serviceRecordHandle);

		/*
		   printf("Service ID: \n");
		   getServiceID(bdAddr, svcRecHandle, &serviceClassUUID);
		   UUID2strn(&serviceClassUUID, UUID_str, MAX_LEN_UUID_STR);
		   printf("  0x%s\n", UUID_str);
		   GFuncPrintServiceClass(&serviceClassUUID, NULL);
		   */

		if (getServiceClassIDList(svcRec, &pGSList) == E_OK) {
			printf("Service Class ID List: \n");
			g_slist_foreach(pGSList, GFuncPrintServiceClass, NULL);
		}

		if (getAccessProtocols(svcRec, &pAccessProtocols) == E_OK) {
			printf("Protocol Descriptor List: \n");
			g_slist_foreach(pAccessProtocols->pSequence, GFuncPrintAccessProtocols, NULL);
		}


		if (getProfileDescriptor(svcRec, &pGSList) == E_OK) {
			printf("Profile Descriptor List: \n");
			g_slist_foreach(pGSList, GFuncPrintProfileDescriptor, NULL);
		}

		if (getBrowseGroupList(svcRec, &pGSList) == E_OK) {
			printf("Browse Group List: \n");
			g_slist_foreach(pGSList, GFuncPrintGroups, NULL);
		}

		if (isGroup(svcRec)) {
			UUID groupId;

			// printf("SvcRec : 0x%x is a group\n", svcRecHandle);
			printf("This is a group.\n");
			groupIDStatus = getGroupID(svcRec, &groupId);
			// printf("Get Group ID status is %d\n", groupIDStatus);
			UUID2strn(&groupId, UUID_str, MAX_LEN_UUID_STR);
			printf("  0x%s\n", UUID_str);
			if (groupId.value.uuid16Bit != groupToExtract->value.uuid16Bit) {
				printf("Extracting it\n");
				extractGroups(&groupId);
			}
		}
	}
	
	return status;
}

int svcSearch(GSList *searchList)
{
	int status = -1, i;
	uint16_t count = 0;
	GSList *pSeq = NULL;
	GSList *attrIdList = NULL;
	uint32_t range = 0x0000ffff;
	// uint32_t range = 0x00000004;
	// uint32_t range = 0x00000100;
	int groupIDStatus = -1;

	// UUID serviceClassUUID;
	GSList *pGSList = NULL;
	AccessProtocols *pAccessProtocols = NULL;

	// printf("In extractGroups method\n");
	attrIdList = g_slist_append(attrIdList, &range);
	status = __SDPServiceSearchAttributeRequest(srvHandle, searchList, 
			RangeOfAttributes, attrIdList, 65535, &pSeq, &count);
	//printf("Status : %d Count : %d\n", status, count);

	if (status || count == 0)
		return status;

	for (i = 0; i < (int)g_slist_length(pSeq); i++) {
		ServiceRecord	*svcRec;
		printf("==============================\n");

		svcRec = (ServiceRecord *)g_slist_nth_data(pSeq, i);
		printSDPDataAsServiceRecord(svcRec);
		printf("------------------------------\n");

		printf("SvcRecHdl: 0x%x\n", svcRec->serviceRecordHandle);

		if (getServiceClassIDList(svcRec, &pGSList) == E_OK) {
			printf("Service Class ID List: \n");
			g_slist_foreach(pGSList, GFuncPrintServiceClass, NULL);
		}

		if (getAccessProtocols(svcRec, &pAccessProtocols) == E_OK) {
			printf("Protocol Descriptor List: \n");
			g_slist_foreach(pAccessProtocols->pSequence, GFuncPrintAccessProtocols, NULL);
		}


		if (getProfileDescriptor(svcRec, &pGSList) == E_OK) {
			printf("Profile Descriptor List: \n");
			g_slist_foreach(pGSList, GFuncPrintProfileDescriptor, NULL);
		}

		if (isGroup(svcRec)) {
			UUID groupId;

			printf("This is a group.\n");
			groupIDStatus = getGroupID(svcRec, &groupId);
			// printf("Get Group ID status is %d\n", groupIDStatus);
			UUID2strn(&groupId, UUID_str, MAX_LEN_UUID_STR);
			printf("  0x%s\n", UUID_str);
		}
	}

	return status;
}


int cmd_browse(struct btctl_command *cmd)
{
	int	status = -1;
	UUID 	uuid;


	//set debug
	//btlog_mask = BTDEBUG_MODULE_SDP;
	//btdaemon = 1;

	status = _openConnection(argv[argind]);
	if (status != -1) {
		makeUUIDFrom16Bits(&uuid, PublicBrowseGroup);
		status = extractGroups(&uuid);
		_closeConnection();

	} else {
		printf("Connection to SDP server failed.\n");
	}
	return status;
}

int cmd_search(struct btctl_command *cmd)
{
	int		status = -1, i;
	GSList		*svcSearchList = NULL;
	UUID 		uuid;
	UUID16Bit	searchArray[] = {
		LANAccessUsingPPPServiceClassID,
		DialupNetworkingServiceClassID,
		ObexObjectPushServiceClassID,
		ObexFileTransferServiceClassID,
		FaxServiceClassID,
		SerialPortServiceClassID,
		IrMCSyncServiceClassID,
		IrMCSyncCommandServiceClassID,
		HeadsetServiceClassID,
		HeadsetAudioGatewayServiceClassID,
		AGHandsFreeServiceClassID,
		HandsFreeServiceClassID
	};


	//set debug
	//btlog_mask = BTDEBUG_MODULE_SDP;
	//btdaemon = 1;

	switch (cmd->cmd) {
		case 1: {
				char str[32];

				status = get_bda(&bdAddr, argv[argind]);
				if (status) {
					printf("Incorrect address given\n");
					return 1;
				}
				bda2str(str, &bdAddr);
				printf("Connection to : %s\n", str);
				status = SDPServiceSearchPortRequest(bdAddr, SerialPortServiceClassID);
				printf("Status/Port: %d\n", status);
				return 0;
			}
	} 
	
	status = _openConnection(argv[argind]);
	if (status == -1) {
		printf("Connection to SDP server failed.\n");
		return status;
	}
	
	for (i = 0; i < (sizeof(searchArray)/sizeof(searchArray[0])); i++) {
		svcSearchList = NULL;
		makeUUIDFrom16Bits(&uuid, searchArray[i]);
		svcSearchList = g_slist_append(svcSearchList, &uuid);
		status = svcSearch(svcSearchList);
		g_slist_free(svcSearchList);
	}
	_closeConnection();

	return status;
}


// FIXME: Remove LISTS!!!
