/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: btctl.h,v 1.3 2002/07/31 14:20:22 kds Exp $

   Header file for the BT server user space daemon 

   Fixes:
   		Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
*/		

#ifndef BTCTL_H
#define BTCTL_H

#if defined(CONFIG_AFFIX_SDP)
#include <affix/sdp.h>
#include <affix/sdpclt.h>
#endif

extern int	argind;
extern char	btdev[IFNAMSIZ];
//extern int	linkmode;
extern int	sdpmode;
extern int	ftpmode;
extern char	cachefile[80];


struct btctl_command {
	char	*name;
	int	(*func)(struct btctl_command *cmd);
	int	cmd;
	char	*arg;
	char	*msg;	/* extra info message */
};

/* Inquiry Cache support */
#define ICSTATE_RANGE	0x01
#define ICSTATE_GONE	0x02
#define ICSTATE_UNKNOWN	0x04
#define ICSTATE_ALL	0xFF

struct inquiry_cache {
	struct inquiry_cache	*next;

	int		id;
	BD_ADDR		bda;
	uint32_t	class;
	char		name[248];
	int		state;		/* in a range */
};

struct inquiry_cache_head {
	struct inquiry_cache	*next;
	int			count;
};

/* btctl.c */
int print_hci_error(int err);
void usage(void);
void print_help(struct btctl_command *cmd);
void print_usage(struct btctl_command *cmd);
int cmd_help(struct btctl_command *bcmd);
int cmd_prompt(struct btctl_command *bcmd);

/* btctl-sdp.c */
int cmd_browse(struct btctl_command *bcmd);
int cmd_search(struct btctl_command *bcmd);

/* btctl-obex.c */
int cmd_open(struct btctl_command *bcmd);
int cmd_close(struct btctl_command *bcmd);
int cmd_ls(struct btctl_command *bcmd);
int cmd_get(struct btctl_command *bcmd);
int cmd_put(struct btctl_command *bcmd);
int cmd_push(struct btctl_command *bcmd);
int cmd_rm(struct btctl_command *bcmd);
int cmd_cd(struct btctl_command *bcmd);
int cmd_mkdir(struct btctl_command *bcmd);

/* btctl-util.c */
void inquiry_cache_retire(void);
void inquiry_cache_print(int state);
void inquiry_cache_free(void);
int inquiry_cache_resolve(BD_ADDR *bda, int id);
struct inquiry_cache *inquiry_cache_lookup(BD_ADDR bda);
int inquiry_cache_del(struct inquiry_cache *node);
struct inquiry_cache *inquiry_cache_add(BD_ADDR bda, uint32_t class, char *name);
void inquiry_cache_load(void);
int inquiry_cache_save(void);

int get_bda(BD_ADDR *bda, char *arg);
int parse_cod(char *buf, uint32_t  cod);
int assign_cod(__u32 *cod);

/* btctl-uart.c */
int cmd_uart(struct btctl_command *bcmd);

/* btctl-audio.c */
int cmd_audio(struct btctl_command *bcmd);
int cmd_play(struct btctl_command *bcmd);
int cmd_listen(struct btctl_command *bcmd);
int cmd_record(struct btctl_command *bcmd);
int cmd_addsco(struct btctl_command *bcmd);

/* PAN */
int cmd_pan(struct btctl_command *bcmd);



#endif	/*	BTCTL_H */

