%{
/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   */

/* 
   $Id: btsrv-lexer.l,v 1.3 2002/07/18 15:14:39 kds Exp $

   Lexical Analizer

	Fixes:	Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
*/

#include <stdio.h>

#include "btsrv.h"
#include "btsrv-parser.h"

int yywrap(void);

struct keyword	kwords[] = {
	{ "device", DEVICE },
	{ "service", SERVICE },
	{ "profile", OPT_PROFILE },
	{ "flags", OPT_FLAGS },
	{ "exec" , OPT_EXEC },
	{ NULL, 0 }
};

int find_keyword(struct keyword *list, char *str);

char	str[128];

extern int waitsemi;

%}

%option yylineno
%array

hex		0x[0-9a-fA-F]+
comment		\#.*\n
word		[a-zA-Z][a-zA-Z0-9]*
wordlist	{word}([ ]*\,[ ]*{word})*
string		\".*\"


%%

[ \t]+	;

{comment}	;

{hex}		{ sscanf(yytext, "%x", &yylval.num); return NUM; }

{word}		{
			int	token;
			/* try to find keyword */
			token = find_keyword(kwords, yytext);
			if (token != -1)
				return token;
			/* just word */
			yylval.str = yytext;
			return WORD;
		}

{wordlist}	{ 
			yylval.str = yytext;
			return WORDLIST;
		}

{string}	{
			yylval.str = yytext+1;
			yytext[yyleng-1] = '\0';
			return STRING;
		}
	
[;\n]		{ 
			if (waitsemi) {
				waitsemi = 0;
				return ';';
			}
		}

.		{ return *yytext; }

%%

int yywrap(void)
{
	return 1;
}

int find_keyword(struct keyword *list, char *str)
{
	for (;list->str; list++) {
		if (strcmp(list->str, str) == 0)
			return list->token;
	}

	return -1;
}

