/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Imre Deak <ext-imre.deak@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: btsrv-sdpreg.c,v 1.2 2002/08/01 17:52:07 kds Exp $

   Registering SDP records with the local SDP server.  

   Fixes:	Imre Deak <ext-imre.deak@nokia.com>
   		Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
*/

#include <affix/config.h>

#include <stdlib.h>
#include <string.h>

#include <affix/btcore.h>

#include <affix/sdp.h>
#include <affix/sdpclt.h>
#include <affix/sdpsrv.h>

#include "btsrv.h"

int sdpreg_init(void)
{
	// initialize the SDP internal data structures

        if (SDPInit(SDP_SVC_PROVIDER) != 0)
                return -1;
	return 0;
}

void sdpreg_cleanup(void)
{
	SDPCleanup();
}

int sdpreg_serialport(service_t *svc)
{
        int status = -1;
        ServiceRecord *svcRec;
        GSList  *browseGroupList = NULL;
        UUID    *pBrowseRootUUID;
        UUID    uuidSerialPort;
        GSList  *serialPortSvcClassIdList = NULL;
        UUID16Bit serialPortSvcClassId16Bit;
        UUID16Bit profileClassId16Bit;
        GSList    *profSeqList = NULL;
        ProfileDescriptor *profDesc;
        GSList    *pSeq = NULL;
 
        /*
         ** First create a service record handle
         */
        svcRec = createServiceRecord();
        if (svcRec == NULL)
        {
                BTERROR("createServiceRecord failed");
                return -1;
        }
 
        serialPortSvcClassId16Bit = SerialPortServiceClassID;
        makeUUIDFrom16Bits(&uuidSerialPort, serialPortSvcClassId16Bit);
        serialPortSvcClassIdList = g_slist_append(serialPortSvcClassIdList,&uuidSerialPort);
        status = setServiceClassID(svcRec, serialPortSvcClassIdList);
        if (status != 0)
        {
                BTERROR("setServiceClassID failed");
                return -1;
        }
        profileClassId16Bit = SerialPortServiceClassID;
        profDesc = (ProfileDescriptor *)malloc(sizeof(ProfileDescriptor));
        if (profDesc == NULL)
        {
                BTERROR("No memory for ProfileDescriptor");
                return -1;
        }
        profDesc->vnum = 0x0100;
        makeUUIDFrom16Bits(&profDesc->profileUUID, profileClassId16Bit);
        profSeqList = g_slist_append(profSeqList, profDesc);
        if (setBTProfileDescriptorList(svcRec, profSeqList) != 0)
        {
                BTERROR("setBTProfileDescriptorList failed");
                return -1;
        }
 
        pBrowseRootUUID = (UUID *)malloc(sizeof(UUID));
        memset(pBrowseRootUUID, 0, sizeof(UUID));
        makeUUIDFrom16Bits(pBrowseRootUUID, PublicBrowseGroup);
	browseGroupList = g_slist_append(browseGroupList, pBrowseRootUUID);
        if (setBrowseGroupList(svcRec, browseGroupList) != 0)
        {
                BTERROR("setBrowseGroupList failed");
                return -1;
        }
 
        {
                AccessProtocols ap;
                ProtocolDescriptor *pDesc, *pDescNext;
                pDesc = (ProtocolDescriptor *)malloc(sizeof(ProtocolDescriptor));
                makeUUIDFrom16Bits(&pDesc->protocolId, L2CAP_UUID);
                pDesc->portNumber = 0xffff;
                pDesc->versionNumber = 0xffff;
 
                pDescNext = (ProtocolDescriptor *)malloc(sizeof(ProtocolDescriptor));
                makeUUIDFrom16Bits(&pDescNext->protocolId, RFCOMM_UUID);
                pDescNext->portNumber = svc->server_ch;
                pDescNext->portNumberSize = 1;
                pDescNext->versionNumber = 0xffff;
 
                ap.pSequence =NULL;
                pSeq = g_slist_append(pSeq, pDesc);
                pSeq = g_slist_append(pSeq, pDescNext);
                ap.pSequence = g_slist_append(ap.pSequence,pSeq);
                if (setAccessProtocols(svcRec, &ap) != 0)
                {
                        BTERROR("setAccessProtocols failed");
                        return -1;
                }
 
        }
 
        if (setInformationalAttributes(
                           svcRec,
                           svc->name,
                           NULL,
                           NULL) != 0)
        {
                BTERROR("setInformationalAttributes failed");
                return -1;
        }
 
        if (registerServiceRecord(svcRec) != 0)
        {
                BTERROR("registerServiceRecord failed");
                return -1;
        }
 
        svc->svcRec = svcRec;

	return 0;
}

int sdpreg_dialupnetworking(service_t *svc)
{
        int status = -1;
        ServiceRecord *svcRec;
        GSList  *browseGroupList = NULL;
        UUID    *pBrowseRootUUID;
        UUID    uuidDialUpNetworking, uuidGenericNetworking;
        GSList  *dialUpSvcClassIdList = NULL;
        UUID16Bit dialUpNetSvcClassId16Bit;
        UUID16Bit uuidGenNetSvcClassId16Bit;
        UUID16Bit profileClassId16Bit;
        GSList    *profSeqList = NULL;
        ProfileDescriptor *profDesc;
        GSList    *pSeq = NULL;
 
        /*
         ** First create a service record handle
         */
        svcRec = createServiceRecord();
        if (svcRec == NULL)
        {
                BTERROR("createServiceRecord failed");
                return -1;
        }
 
        dialUpNetSvcClassId16Bit = DialupNetworkingServiceClassID;
        makeUUIDFrom16Bits(&uuidDialUpNetworking, dialUpNetSvcClassId16Bit);
        uuidGenNetSvcClassId16Bit = GenericNetworkingServiceClassID;
        makeUUIDFrom16Bits(&uuidGenericNetworking, uuidGenNetSvcClassId16Bit);
        dialUpSvcClassIdList = g_slist_append(dialUpSvcClassIdList,&uuidDialUpNetworking);
        dialUpSvcClassIdList = g_slist_append(dialUpSvcClassIdList,&uuidGenericNetworking);
        status = setServiceClassID(svcRec, dialUpSvcClassIdList);
        if (status != 0)
        {
                BTERROR("setServiceClassID failed");
                return -1;
        }
        profileClassId16Bit = DialupNetworkingServiceClassID;
        profDesc = (ProfileDescriptor *)malloc(sizeof(ProfileDescriptor));
        if (profDesc == NULL)
        {
                BTERROR("No memory for ProfileDescriptor");
                return -1;
        }
        profDesc->vnum = 0x0100;
        makeUUIDFrom16Bits(&profDesc->profileUUID, profileClassId16Bit);
        profSeqList = g_slist_append(profSeqList, profDesc);
        if (setBTProfileDescriptorList(svcRec, profSeqList) != 0)
        {
                BTERROR("setBTProfileDescriptorList failed");
                return -1;
        }
	pBrowseRootUUID = (UUID *)malloc(sizeof(UUID));
        memset(pBrowseRootUUID, 0, sizeof(UUID));
        makeUUIDFrom16Bits(pBrowseRootUUID, PublicBrowseGroup);
        browseGroupList = g_slist_append(browseGroupList, pBrowseRootUUID);
        if (setBrowseGroupList(svcRec, browseGroupList) != 0)
        {
                BTERROR("setBrowseGroupList failed");
                return -1;
        }
 
        {
                AccessProtocols ap;
                ProtocolDescriptor *pDesc, *pDescNext;
                pDesc = (ProtocolDescriptor *)malloc(sizeof(ProtocolDescriptor));
                makeUUIDFrom16Bits(&pDesc->protocolId, L2CAP_UUID);
                pDesc->portNumber = 0xffff;
                pDesc->versionNumber = 0xffff;
 
                pDescNext = (ProtocolDescriptor *)malloc(sizeof(ProtocolDescriptor));
                makeUUIDFrom16Bits(&pDescNext->protocolId, RFCOMM_UUID);
                pDescNext->portNumber = svc->server_ch;
                pDescNext->portNumberSize = 1;
                pDescNext->versionNumber = 0xffff;
 
                ap.pSequence =NULL;
                pSeq = g_slist_append(pSeq, pDesc);
                pSeq = g_slist_append(pSeq, pDescNext);
                ap.pSequence = g_slist_append(ap.pSequence,pSeq);
                if (setAccessProtocols(svcRec, &ap) != 0)
                {
                        BTERROR("setAccessProtocols failed");
                        return -1;
                }
 
        }
 
        if (setInformationalAttributes(
                           svcRec,
                           svc->name,
                           NULL,
                           NULL) != 0)
        {
                BTERROR("setInformationalAttributes failed");
                return -1;
        }
 
        if (registerServiceRecord(svcRec) != 0)
        {
                BTERROR("registerServiceRecord failed");
                return -1;
        }
 
        svc->svcRec = svcRec;

        return 0;
}

int sdpreg_lanaccess(service_t *svc)
{
	int status = -1;
	ServiceRecord *svcRec;

	GSList *browseGroupList = NULL;
	UUID *pBrowseRootUUID;

	/* 
	 ** First create a service record handle
	 */
	svcRec = createServiceRecord();
	if (svcRec == NULL)
		return -1;

	{
		UUID uuidLANAccess, uuidGenericNetworking;
		GSList *LANAccessSvcClassIdList = NULL;
		UUID16Bit uuidGenNetSvcClassId16Bit;

		UUID16Bit uuidLANAccessSvcClassId16Bit = LANAccessUsingPPPServiceClassID;
		makeUUIDFrom16Bits(&uuidLANAccess, uuidLANAccessSvcClassId16Bit);
		uuidGenNetSvcClassId16Bit = GenericNetworkingServiceClassID;
		makeUUIDFrom16Bits(&uuidGenericNetworking, uuidGenNetSvcClassId16Bit);

		LANAccessSvcClassIdList = g_slist_append(LANAccessSvcClassIdList,&uuidLANAccess);
		LANAccessSvcClassIdList = g_slist_append(LANAccessSvcClassIdList,&uuidGenericNetworking);

		status = setServiceClassID(svcRec, LANAccessSvcClassIdList);


	}

	{
		UUID16Bit profileClassId16Bit = LANAccessUsingPPPServiceClassID;
		GSList *profSeqList = NULL;
		ProfileDescriptor *profDesc =
			(ProfileDescriptor *)malloc(sizeof(ProfileDescriptor));
		profDesc->vnum = 0x0100;
		makeUUIDFrom16Bits(&profDesc->profileUUID, profileClassId16Bit);
		profSeqList = g_slist_append(profSeqList, profDesc);
		status = setBTProfileDescriptorList(svcRec, profSeqList);
		if (status != 0)
			return -1;
	}

	{
		pBrowseRootUUID = (UUID *)malloc(sizeof(UUID));
		memset(pBrowseRootUUID, 0, sizeof(UUID));
		makeUUIDFrom16Bits(pBrowseRootUUID, PublicBrowseGroup);
		browseGroupList = g_slist_append(browseGroupList, pBrowseRootUUID);
		status = setBrowseGroupList(svcRec, browseGroupList);
		if (status != 0)
			return -1;
	}

	{
		GSList *pSeq = NULL;
		AccessProtocols ap;
		ProtocolDescriptor *pDesc, *pDescNext;
		pDesc = (ProtocolDescriptor *)malloc(sizeof(ProtocolDescriptor));
		makeUUIDFrom16Bits(&pDesc->protocolId, L2CAP_UUID);
		pDesc->portNumber = 0xffff;
		pDesc->versionNumber = 0xffff;

		pDescNext = (ProtocolDescriptor *)malloc(sizeof(ProtocolDescriptor));
		makeUUIDFrom16Bits(&pDescNext->protocolId, RFCOMM_UUID);
		pDescNext->portNumber = svc->server_ch;
		pDescNext->portNumberSize = 1;
		pDescNext->versionNumber = 0xffff;

		ap.pSequence = NULL;
		pSeq = g_slist_append(pSeq, pDesc);
		pSeq = g_slist_append(pSeq, pDescNext);
		ap.pSequence = g_slist_append(ap.pSequence,pSeq);
		status = setAccessProtocols(svcRec, &ap);
		if (status != 0)
			return -1;
	}

	status = setInformationalAttributes(
			svcRec,
			svc->name,
			NULL,
			NULL);
	if (status != 0)
		return -1;

	status = registerServiceRecord(svcRec);
	if (status != 0)
		return -1;

        svc->svcRec = svcRec;

	return 0;
}


int sdpreg_obex_filetransfer(service_t *svc)
{
   int status = -1;
   ServiceRecord *svcRec;
 
   GSList *browseGroupList = NULL;
   UUID *pBrowseRootUUID;
 
   // initialize the SDP internal data structures
//   status = SDPInit();
 
//   if (status != 0)
 //       return -1;
 
      /*
      ** First create a service record handle
      */
      svcRec = createServiceRecord();
      if (svcRec == NULL)
        return -1;
 
      {
 
 
         UUID uuidObex;
         GSList *obexSvcClassIdList = NULL;
 
         UUID16Bit uuidObexSvcClassId16Bit = ObexFileTransferServiceClassID;
         makeUUIDFrom16Bits(&uuidObex, uuidObexSvcClassId16Bit);
         obexSvcClassIdList = g_slist_append(obexSvcClassIdList,&uuidObex);
 
         status = setServiceClassID(svcRec, obexSvcClassIdList);
 
 
     }
 
     {
         UUID16Bit profileClassId16Bit = ObexFileTransferServiceClassID;
         GSList *profSeqList = NULL;
         ProfileDescriptor *profDesc =
                 (ProfileDescriptor *)malloc(sizeof(ProfileDescriptor));
         profDesc->vnum = 0x0100;
         makeUUIDFrom16Bits(&profDesc->profileUUID, profileClassId16Bit);
         profSeqList = g_slist_append(profSeqList, profDesc);
         status = setBTProfileDescriptorList(svcRec, profSeqList);
         if (status != 0)
                return -1;
     }
 
     {
         pBrowseRootUUID = (UUID *)malloc(sizeof(UUID));
        memset(pBrowseRootUUID, 0, sizeof(UUID));
        makeUUIDFrom16Bits(pBrowseRootUUID, PublicBrowseGroup);
         browseGroupList = g_slist_append(browseGroupList, pBrowseRootUUID);
         status = setBrowseGroupList(svcRec, browseGroupList);
         if (status != 0)
                return -1;
     }

     {
         GSList *pSeq = NULL;
         AccessProtocols ap;
         ProtocolDescriptor *pDesc, *pDescNext, *pObexDesc;
         pDesc = (ProtocolDescriptor *)malloc(sizeof(ProtocolDescriptor));
         makeUUIDFrom16Bits(&pDesc->protocolId, L2CAP_UUID);
         pDesc->portNumber = 0xffff;
         pDesc->versionNumber = 0xffff;
 
         pDescNext = (ProtocolDescriptor *)malloc(sizeof(ProtocolDescriptor));
         makeUUIDFrom16Bits(&pDescNext->protocolId, RFCOMM_UUID);
         pDescNext->portNumber = svc->server_ch;
         pDescNext->portNumberSize = 1;
         pDescNext->versionNumber = 0xffff;
 
         pObexDesc = (ProtocolDescriptor *)malloc(sizeof(ProtocolDescriptor));
         makeUUIDFrom16Bits(&pObexDesc->protocolId, OBEX_UUID);
         pObexDesc->portNumber = 0xffff;
         pObexDesc->versionNumber = 0xffff;
 
         ap.pSequence = NULL;
         pSeq = g_slist_append(pSeq, pDesc);
         pSeq = g_slist_append(pSeq, pDescNext);
         pSeq = g_slist_append(pSeq, pObexDesc);
         ap.pSequence = g_slist_append(ap.pSequence,pSeq);
         status = setAccessProtocols(svcRec, &ap);
         if (status != 0)
                return -1;
      }
 
         status = setInformationalAttributes(
                           svcRec,
                           svc->name,
                           NULL,
                           NULL);
         if (status != 0)
                return -1;
 
        status = registerServiceRecord(svcRec);
        if (status != 0)
                return -1;
 
        svc->svcRec = svcRec;

        return 0;
}

int sdpreg_obex_objectpush(service_t *svc)
{
   int status = -1;
   ServiceRecord *svcRec;
 
   GSList *browseGroupList = NULL;
   UUID *pBrowseRootUUID;
 
   // initialize the SDP internal data structures
//   status = SDPInit();
 
//   if (status != 0)
//        return -1;
 
      /*
      ** First create a service record handle
      */
      svcRec = createServiceRecord();
      if (svcRec == NULL)
        return -1;
 
      {
 
 
         UUID uuidObex;
         GSList *obexSvcClassIdList = NULL;
 
         UUID16Bit uuidObexSvcClassId16Bit = ObexObjectPushServiceClassID;
         makeUUIDFrom16Bits(&uuidObex, uuidObexSvcClassId16Bit);
         obexSvcClassIdList = g_slist_append(obexSvcClassIdList,&uuidObex);
 
         status = setServiceClassID(svcRec, obexSvcClassIdList);
 
 
     }
 
     {
         UUID16Bit profileClassId16Bit = ObexObjectPushServiceClassID;
         GSList *profSeqList = NULL;
         ProfileDescriptor *profDesc =
                 (ProfileDescriptor *)malloc(sizeof(ProfileDescriptor));
         profDesc->vnum = 0x0100;
         makeUUIDFrom16Bits(&profDesc->profileUUID, profileClassId16Bit);
         profSeqList = g_slist_append(profSeqList, profDesc);
         status = setBTProfileDescriptorList(svcRec, profSeqList);
         if (status != 0)
                return -1;
     }
 
     {
         pBrowseRootUUID = (UUID *)malloc(sizeof(UUID));
        memset(pBrowseRootUUID, 0, sizeof(UUID));
        makeUUIDFrom16Bits(pBrowseRootUUID, PublicBrowseGroup);
         browseGroupList = g_slist_append(browseGroupList, pBrowseRootUUID);
         status = setBrowseGroupList(svcRec, browseGroupList);
         if (status != 0)
                return -1;
     }

     {
         GSList *pSeq = NULL;
         AccessProtocols ap;
         ProtocolDescriptor *pDesc, *pDescNext, *pObexDesc;
         pDesc = (ProtocolDescriptor *)malloc(sizeof(ProtocolDescriptor));
         makeUUIDFrom16Bits(&pDesc->protocolId, L2CAP_UUID);
         pDesc->portNumber = 0xffff;
         pDesc->versionNumber = 0xffff;
 
         pDescNext = (ProtocolDescriptor *)malloc(sizeof(ProtocolDescriptor));
         makeUUIDFrom16Bits(&pDescNext->protocolId, RFCOMM_UUID);
         pDescNext->portNumber = svc->server_ch;
         pDescNext->portNumberSize = 1;
         pDescNext->versionNumber = 0xffff;
 
         pObexDesc = (ProtocolDescriptor *)malloc(sizeof(ProtocolDescriptor));
         makeUUIDFrom16Bits(&pObexDesc->protocolId, OBEX_UUID);
         pObexDesc->portNumber = 0xffff;
         pObexDesc->versionNumber = 0xffff;
 
         ap.pSequence = NULL;
         pSeq = g_slist_append(pSeq, pDesc);
         pSeq = g_slist_append(pSeq, pDescNext);
         pSeq = g_slist_append(pSeq, pObexDesc);
         ap.pSequence = g_slist_append(ap.pSequence,pSeq);
         status = setAccessProtocols(svcRec, &ap);
         if (status != 0)
                return -1;
      }
 
         status = setInformationalAttributes(
                           svcRec,
                           svc->name,
                           NULL,
                           NULL);
         if (status != 0)
                return -1;
 
        status = registerServiceRecord(svcRec);
        if (status != 0)
                return -1;
 
        svc->svcRec = svcRec;

        return 0;
}

int sdpreg_register(service_t *svc)
{
	if ( svc->profile_reg_func &&
			svc->profile_reg_func(svc) < 0)
	{
		BTERROR("Unable to register SDP record for service %s", svc->name);
		return -1;
	}

	return 0;
}

int sdpreg_unregister(service_t *svc)
{
	int	status;
	ServiceRecord	*svcRec = svc->svcRec;
	
	if (svcRec) {
		status = deleteServiceRecord(svcRec);
		switch (status) {
			case E_OK:
				BTINFO("service %s deleted", svc->name);
				break;
			case E_INVALID_ARGS:
				BTERROR("Error: You cannot delete service %s", svc->name);
				break;
			default:
				BTERROR("Something went wrong. deleteServiceRecord() returned: %d\n", status);
		}
	}
 
	return 0;
}
