/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Imre Deak <ext-imre.deak@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: btsrv-util.c,v 1.3 2002/07/19 20:51:14 kds Exp $

   Utility functions to read and parse the BT server config file. 

   Fixes:	Imre Deak <ext-imre.deak@nokia.com>
   		Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
*/

#include <affix/config.h>

#include <stdio.h>
#include <stdlib.h>
#include <sys/fcntl.h>
#include <sys/mman.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>

#include <affix/btcore.h>

#include "btsrv.h"

char *btsrv_format_cmd(const char *cmd, int conid, int line,
		       char *bd_addr, int srv_ch)
{
        char    *formatted_cmd;
	int	i, j, res;
 
        formatted_cmd = (char *)malloc(MAX_CMD_LEN + 1);
        if (formatted_cmd == NULL)
                return NULL;
        i = j = 0;
        while (cmd[i])
        {
                switch (cmd[i])
                {
                case '%':
                        switch(cmd[i + 1])
                        {
                        case 'a':
                                res = snprintf(&formatted_cmd[j],
					      MAX_CMD_LEN - j,
					      "%02x:%02x:%02x:%02x:%02x:%02x",
                                        	bd_addr[0], bd_addr[1],
						bd_addr[2], bd_addr[3],
                                        	bd_addr[4], bd_addr[5]);
                                break;
                        case 'l':
                                res = snprintf(&formatted_cmd[j],
					      MAX_CMD_LEN - j, "%d", line);
                                break;
			case 'i':
                                res = snprintf(&formatted_cmd[j],
					      MAX_CMD_LEN - j, "%d", conid);
                                break;

                        case 'c':
                                res = snprintf(&formatted_cmd[j],
					       MAX_CMD_LEN - j, "%d", srv_ch);
                                break;
                        case '%':
                                formatted_cmd[j] = '%';
				res = j < MAX_CMD_LEN ? 1 : -1;
                                break;
			default:
				res = -1;
                        }
                        i++;	/* skip % sign */
                        break;
                default:
                        formatted_cmd[j] = cmd[i];
			res = j < MAX_CMD_LEN ? 1 : -1;
                }
		if (res == -1)
		{
			free(formatted_cmd);
			return NULL;
		}
		j += res;
                i++;
        }
        formatted_cmd[j] = '\0';
        return formatted_cmd;
}

int become_daemon(void)
{
	int 	fd;
	
	if (getppid() != 1) {
		signal(SIGTTOU, SIG_IGN);
		signal(SIGTTIN, SIG_IGN);
		signal(SIGTSTP, SIG_IGN);
		if (fork() != 0)
			return 0;	/* exit if parent */
		setsid();
	}
	for (fd = 0; fd < 3; fd++)
		close(fd);
	chdir("/");

	return 1;
}


