/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Imre Deak <ext-imre.deak@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: btsrv.h,v 1.3 2002/08/01 17:52:08 kds Exp $

   Header file for the BT server user space daemon 

   Fixes:	Imre Deak <ext-imre.deak@nokia.com>
   		Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
*/		

#ifndef BTSRV_H
#define BTSRV_H

#if defined(CONFIG_AFFIX_SDP)
#include <affix/sdp.h>
#include <affix/sdpsrv.h>
#endif

#define MAX_SERVICE_NAME_LEN	255
#define MAX_PROFILE_NAME_LEN     50
#define MAX_CMD_LEN             500
#define MAX_SERVICE_NUM		10

#define SRV_FLAG_RFCOMM_TTY	0x01
#define SRV_FLAG_STD		0x02

typedef struct _service_t	service_t;

typedef int (*sdpreg_func_t)(service_t *svc);

struct _service_t{
	/* service stuff */
        int     server_ch;
        char    name[MAX_SERVICE_NAME_LEN + 1];
        sdpreg_func_t	profile_reg_func;
        char    cmd[MAX_CMD_LEN + 1];
	int	svc_fd;
	int	flags;
#if defined(CONFIG_AFFIX_SDP)
	/* SDP stuff */
	ServiceRecord	*svcRec;
#endif
};

extern service_t services[MAX_SERVICE_NUM];

int btsrv_read_config(char *config_file, int *service_num);
char *btsrv_format_cmd(const char *cmd, int conid, int line,
		       char *bd_addr, int srv_ch);
int become_daemon(void);

int sdpreg_init(void);
void sdpreg_cleanup(void);
int sdpreg_register(service_t *svc);
int sdpreg_unregister(service_t *svc);

int sdpreg_serialport(service_t *svc);
int sdpreg_dialupnetworking(service_t *svc);
int sdpreg_lanaccess(service_t *svc);
int sdpreg_obex_filetransfer(service_t *svc);
int sdpreg_obex_objectpush(service_t *svc);

/* for parser */

struct keyword {
	char	*str;
	int	token;
};


#endif	/*	BTSRV_H */

