/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: obex_client.c,v 1.4 2002/07/31 15:04:07 kds Exp $

   OBEX client lib 

   Fixes:	Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

*/

#include <affix/config.h>

#include <glib.h>
#include <string.h>
#include <stdlib.h>
#include <libgen.h>

#include <sys/socket.h>
#include <sys/stat.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>

#include <stdio.h>
#include <unistd.h>
#include <errno.h>

#include <affix/bluetooth.h>
#include <affix/btcore.h>
//#include <affix/debug.h>

// OBEX stuff
#include <openobex/obex.h>
#include "obex_server.h"
#include <affix/obex.h>



#define BT_SERVICE "OBEX"
//char	*obex_root_path="/var/spool/affix";

char	browsing[16] = { 0xF9, 0xEC, 0x7B, 0xC4, 0x95, 0x3C, 0x11, 0xD2,
			0x98, 0x4E, 0x52, 0x54, 0x00, 0xDC, 0x9E, 0x09 };

#if defined(CONFIG_AFFIX_SDP)
UUID16Bit	ServiceID;
#endif
const guint8	*target = NULL;
gint		target_len = 0;


char *obex_error(int rsp)
{
	switch (rsp) {
		case 0:
		case OBEX_RSP_SUCCESS:
			return "Success";
		case OBEX_RSP_NOT_FOUND:
			return "Not Found";
		case OBEX_RSP_FORBIDDEN:
			return "Forbidden";
		case OBEX_RSP_NO_CONTENT:
			return "No Content";
		case OBEX_RSP_BAD_REQUEST:
			return "Bad Reuqest";
		case OBEX_RSP_INTERNAL_SERVER_ERROR:
			return "Internal Server Error";
		case OBEX_RSP_UNAUTHORIZED:
			return "Not authorized";
		default:
			return "Unspecified Error";
	}
}
	
/*
  RFCOMM stuff
*/
int rfcomm_connect(void *addr)
{
	int		err, fd;
	char		str[20];
	BD_ADDR		bda;
	int		sch;
	struct sockaddr_affix	saddr;
	struct sockaddr_affix	*sa = addr;

	bda = sa->bda;

	bda2str(str, &bda);
	BTDEBUG("Connecting to host %s ...\n", str);

#if defined(CONFIG_AFFIX_SDP)
	if (sa->port == 0) {
		sch = SDPServiceSearchPortRequest(bda, ServiceID);
		if (sch > 0)
			BTDEBUG("Service found on channel %d\n", sch);
		else if ( sch == 0 ) {
			BTDEBUG("Service is not available\n");
			return -1;
		} else {
			BTDEBUG("Unable to get service channel: %d\n", sch);
			return -1;
		}
	} else
#endif	
		sch = sa->port;

	fd = socket(PF_AFFIX, SOCK_STREAM, BTPROTO_RFCOMM);
	if (fd < 0) {
		BTDEBUG("Unable to create RFCOMM socket: %d\n", PF_AFFIX);
		return -1;
	}

	saddr.bda = bda;
	saddr.port = sch;
	saddr.local = BDADDR_ANY;
	err = connect(fd, (struct sockaddr*)&saddr, sizeof(saddr));
	if (err < 0) {
		BTDEBUG("Unable to connect to remote side\n");
		close(fd);
		return -1;
	}
	
	return fd;
}

void connect_client(obex_t *handle, obex_object_t *object, gint obex_rsp)
{
	obex_headerdata_t	hv;
	guint8			hi;
	gint			hlen;
	guint8			*nonhdrdata;
	const guint8		*who = NULL;
	gint			who_len = 0;
	obexclt_t	*clt = OBEX_GetUserData(handle);

	BTDEBUG("");

	clt->rsp = obex_rsp;
	if (obex_rsp != OBEX_RSP_SUCCESS)
		return;

	if(OBEX_ObjectGetNonHdrData(object, &nonhdrdata) == 4) {
		BTDEBUG("Version: 0x%02x. Flags: 0x%02x  OBEX packet length:%d",
			nonhdrdata[0], nonhdrdata[1], *(guint16*)&(nonhdrdata[2]));
	} else {
		BTERROR("Invalid packet content.");
	}
	while(OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, &hlen)) {
		switch (hi) {
		case OBEX_HDR_WHO:
			who = hv.bs;
			who_len = hlen;
			BTDEBUG("got WHO: %*s, len: %d", who_len, who, who_len);
			break;
		case OBEX_HDR_CONNECTION:
			clt->con_id = hv.bq4;
			BTDEBUG("got Conection ID: %#x\n", hv.bq4);
			break;
		default:	
			BTDEBUG(" Skipped header %02x", hi);
		}
	}

	if (who_len == 6)	{
		if(strncmp("Linux", who, 6) == 0)	{
			BTDEBUG("Weeeha. I'm talking to the coolest OS ever!");
		}
	}
}

void get_client(obex_t *handle, obex_object_t *object, gint obex_rsp)
{
	obex_headerdata_t	hv;
	guint8			hi;
	gint			hlen;
	char			*body = NULL;
	int			body_len = 0;
	int			len;
	obexclt_t	*clt = OBEX_GetUserData(handle);

	BTDEBUG("");

	clt->rsp = obex_rsp;
	if (obex_rsp != OBEX_RSP_SUCCESS)
		return;

	while(OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, &hlen)) {
		switch (hi) {
		case OBEX_HDR_LENGTH:
			len = hv.bq4;
			BTDEBUG("got LENGTH");
			break;
		case OBEX_HDR_BODY:
			//BTDEBUG("%*s\n", hlen, hv.bs);
			body = (char*)hv.bs;
			body_len = hlen;
			BTDEBUG("got BODY");
			break;
		case OBEX_HDR_BODY_END:
			BTDEBUG("got BODYEND");
			break;
		case OBEX_HDR_CONNECTION:
			BTDEBUG("got CONNECTION");
			break;
		case OBEX_HDR_NAME:
			BTDEBUG("got NAME");
			break;
		
		default:	
			BTDEBUG(" Skipped header %02x", hi);
		}
	}

	switch (clt->opcode) {
		case OBEX_LS:
			memcpy(clt->buf, body, body_len); 
			break;
		case OBEX_GET:
			if (body) {
				BTDEBUG("File %s (%d bytes) received, saving ...\n", clt->buf, body_len);
				safe_save_file(clt->buf, body, body_len);
			}
			break;
	}
}

void put_client(obex_t *handle, obex_object_t *object, gint obex_rsp)
{
	obex_headerdata_t	hv;
	guint8			hi;
	gint			hlen;
	obexclt_t	*clt = OBEX_GetUserData(handle);

	int	len;

	BTDEBUG("");

	clt->rsp = obex_rsp;
	if (obex_rsp != OBEX_RSP_SUCCESS)
		return;

	while(OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, &hlen)) {
		switch (hi) {
		case OBEX_HDR_LENGTH:
			len = hv.bq4;
			BTDEBUG("got LENGTH");
			break;
		case OBEX_HDR_CONNECTION:
			BTDEBUG("got CONNECTION");
			break;
		case OBEX_HDR_NAME:
			BTDEBUG("got NAME");
			break;

		default:
			BTDEBUG(" Skipped header %02x", hi);
		}
	}
}

void setpath_client(obex_t *handle, obex_object_t *object, gint obex_rsp)
{
	obex_headerdata_t	hv;
	guint8			hi;
	gint			hlen;
	obexclt_t	*clt = OBEX_GetUserData(handle);

	BTDEBUG("");

	clt->rsp = obex_rsp;
	if (obex_rsp != OBEX_RSP_SUCCESS)
		return;

	while(OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, &hlen)) {
		switch (hi) {
		case OBEX_HDR_CONNECTION:
			BTDEBUG("got CONNECTION");
			break;
		case OBEX_HDR_NAME:
			BTDEBUG("got NAME");
			break;

		default:
			BTDEBUG(" Skipped header %02x", hi);
		}
	}
}


void request_done(obex_t *handle, obex_object_t *object, gint obex_cmd, gint obex_rsp)
{
	obexclt_t *clt = OBEX_GetUserData(handle);

	BTDEBUG("Command (%02x) has now finished, rsp: %02x", obex_cmd, obex_rsp);

	switch (obex_cmd) {
	case OBEX_CMD_DISCONNECT:
		BTDEBUG("Disconnect done!");
		OBEX_TransportDisconnect(handle);
		clt->clientdone = TRUE;
		break;
	case OBEX_CMD_CONNECT:
		BTDEBUG("Connected!\n");
		connect_client(handle, object, obex_rsp);
		clt->clientdone = TRUE;
		break;
	case OBEX_CMD_GET:
		get_client(handle, object, obex_rsp);
		clt->clientdone = TRUE;
		break;
	case OBEX_CMD_PUT:
		put_client(handle, object, obex_rsp);
		clt->clientdone = TRUE;
		break;
	case OBEX_CMD_SETPATH:
		setpath_client(handle, object, obex_rsp);
		clt->clientdone = TRUE;
		break;

	default:
		BTDEBUG("Command (%02x) has now finished", obex_cmd);
		break;
	}
}


int handle_response(obexclt_t *clt, char *service)
{
	int	err = 0;

	clt->clientdone = FALSE;
	while (!clt->clientdone) {
		if( (err = OBEX_HandleInput(clt->handle, 1)) < 0) {
			BTERROR("Error while doing OBEX_HandleInput()");
			break;
		}
		err = (clt->rsp == OBEX_RSP_SUCCESS)?0:clt->rsp;
	}
	return err;
}

//
// Add more data to stream.
//
static gint fillstream(obex_t *handle, obex_object_t *object)
{
	gint 			actual;
	obexclt_t		*clt = OBEX_GetUserData(handle);
	obex_headerdata_t	hv;
		
	actual = read(clt->sfd, clt->buf, OBEX_STREAM_CHUNK);
	//BTDEBUG("send %d bytes\n", actual);
	if(actual > 0) {
		/* Read was ok! */
		hv.bs = clt->buf;
		OBEX_ObjectAddHeader(handle, object, OBEX_HDR_BODY,
				hv, actual, OBEX_FL_STREAM_DATA);
	}
	else if(actual == 0) {
		/* EOF */
		hv.bs = clt->buf;
		close(clt->sfd); clt->sfd = -1;
		OBEX_ObjectAddHeader(handle, object, OBEX_HDR_BODY,
				hv, 0, OBEX_FL_STREAM_DATAEND);
	}
        else {
		/* Error */
		hv.bs = NULL;
		close(clt->sfd); clt->sfd = -1;
		OBEX_ObjectAddHeader(handle, object, OBEX_HDR_BODY,
				hv, 0, OBEX_FL_STREAM_DATA);
	}

	return actual;
}

//
// Read data from stream.
//
static gint readstream(obex_t *handle, obex_object_t *object)
{
	gint 		actual;
	obexclt_t	*clt = OBEX_GetUserData(handle);
	const guint8	*buf;
	gint		len;
		
	actual = OBEX_ObjectReadStream(handle, object, &buf);
	if (actual > 0) {
		len = write(clt->sfd, buf, actual);
	} else if (actual == 0) {
		//close(clt->sfd); clt->sfd = -1;
	}

	//BTDEBUG("Got %d bytes\n", actual);
	return actual;
}


//
// Called by the obex-layer when some event occurs.
//
void obex_event(obex_t *handle, obex_object_t *object, gint mode, gint event, gint obex_cmd, gint obex_rsp)
{
	switch (event)	{
		case OBEX_EV_PROGRESS:
			//BTDEBUG("Made some progress...");
			break;

		case OBEX_EV_ABORT:
			BTERROR("Request aborted!");
			break;

		case OBEX_EV_REQDONE:
			request_done(handle, object, obex_cmd, obex_rsp);
			break;

		case OBEX_EV_REQHINT:
			/* Accept any command. Not rellay good, but this is a test-program :) */
			//OBEX_ObjectSetRsp(object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
			break;

		case OBEX_EV_REQ:
			BTDEBUG("Command request (%02x)", obex_cmd);
			//server_request(handle, object, event, obex_cmd);
			break;

		case OBEX_EV_LINKERR:
			OBEX_TransportDisconnect(handle);
			BTERROR("Link broken!");
			break;

		case OBEX_EV_STREAMEMPTY:
			fillstream(handle, object);
			break;

		case OBEX_EV_STREAMAVAIL:
			readstream(handle, object);
			break;

		case OBEX_EV_PARSEERR:
			//OBEX_TransportDisconnect(handle);
			BTERROR("Parse Error!");
			break;

		default:
			BTERROR("Unknown event %02x!", event);
			break;
	}
}

obexclt_t *__obex_connect(void *addr, int *err)
{
	obex_object_t		*oo;	// OBEX Object
	obex_headerdata_t	hv;
	obexclt_t		*clt;


	clt = malloc(sizeof(obexclt_t));
	if (clt == NULL)
		return NULL;
	memset(clt, 0, sizeof(obexclt_t));

	if (linkmode != LINK_TCP) {
		if (!(clt->handle = btobex_init(&clt->link, obex_event, 0))) {
			BTERROR( "OBEX_Init failed:%s", strerror(errno));
			exit(0);
		}

		if (linkmode == LINK_BLUETOOTH) {
			// connect to the server
			clt->fd = rfcomm_connect(addr);
			if (clt->fd < 0) {
				BTDEBUG("Unable to connect to Bluetooth OBEX server\n");
				*err = clt->fd;	
				return NULL;
			}

			btobex_setsocket(&clt->link, clt->fd);
		} else {
			btobex_setport(&clt->link, addr);
		}

	} else { // tcpmode
		struct sockaddr_in	saddr;
		struct sockaddr_in	*in_addr = addr;

		BTDEBUG("Using TCP transport: host = %s", inet_ntoa(in_addr->sin_addr));
		if (!(clt->handle = OBEX_Init(OBEX_TRANS_INET, obex_event, 0))) {
			BTERROR( "OBEX_Init failed:%s", strerror(errno));
			*err = -1;
			return NULL;
		}
		
	        saddr = *in_addr;
		// connect to the server
		*err = OBEX_TransportConnect(clt->handle, (struct sockaddr*)&saddr, sizeof(saddr));
		if (*err < 0) {
			BTDEBUG("Unable to connect to the server\n");
			*err = -1;
			return NULL;
		}
	}
	
	OBEX_SetUserData(clt->handle, clt);

	// create new object
	oo = OBEX_ObjectNew(clt->handle, OBEX_CMD_CONNECT);
	if (target) {
		hv.bs = target;
		OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_TARGET, hv, target_len, 0);
	}
	*err = OBEX_Request(clt->handle, oo);
	if (*err)
		goto exit;
	*err = handle_response(clt, BT_SERVICE);
	BTDEBUG("Connection return code: %d, id: %d\n", *err, clt->con_id);
	if (*err)
		goto exit;
	if (target && clt->con_id == 0) {
		*err = OBEX_RSP_CONFLICT;
		goto exit;
	}
	BTDEBUG("Connection established\n");
	return clt;
exit:
	obex_disconnect(clt);
	return NULL;	
}

int obex_disconnect(obexclt_t *clt)
{
	int			err;
	obex_object_t		*oo;	// OBEX Object

	// create new object
	oo = OBEX_ObjectNew(clt->handle, OBEX_CMD_DISCONNECT);
	err = OBEX_Request(clt->handle, oo);
	if (err)
		return err;
	handle_response(clt, BT_SERVICE);

	if (linkmode != LINK_TCP) {
		btobex_cleanup(&clt->link, 1);
		close(clt->fd);
	} else
		OBEX_TransportDisconnect(clt->handle);

	free(clt);
	return 0;
}

int obex_browse(obexclt_t *clt, char *name, char *buf)
{
	int			err;
	obex_object_t		*oo;	// OBEX Object
	obex_headerdata_t	hv;

	clt->opcode = OBEX_LS;
	clt->buf = buf;

	// create new object
	oo = OBEX_ObjectNew(clt->handle, OBEX_CMD_GET);
	hv.bq4 = clt->con_id;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_CONNECTION, hv, 4, 0);
	hv.bs = "x-obex/folder-listing";
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_TYPE, hv, strlen(hv.bs)+1, 0);

	err = OBEX_Request(clt->handle, oo);
	if (!err)
		err = handle_response(clt, BT_SERVICE);

	return err;
}

int obex_put(obexclt_t *clt, char *path)
{
	int			err;
	obex_object_t		*oo;	// OBEX Object
	obex_headerdata_t	hv;
	int			file_size;
	gchar 			*ucname;
	int			ucname_len;
	char			*name, *pathc;

	clt->opcode = OBEX_PUT;

	pathc = strdup(path);
	name = basename(pathc);

	clt->sfd = open(path, O_RDONLY, 0);
	if (clt->sfd < 0) {
		BTDEBUG("File not found: %s\n", pathc);
		free(pathc);
		return -1;
	}

	file_size = get_filesize(path);

	clt->buf = g_malloc(OBEX_STREAM_CHUNK);
	if (!clt->buf) {
		close(clt->sfd); clt->sfd = -1;
		free(pathc);
		return -1;	
	}

	ucname_len = (strlen(name)+1)<<1;
	ucname = g_malloc(ucname_len);
	if (!ucname){
		g_free(clt->buf); clt->buf = NULL;
		close(clt->sfd); clt->sfd = -1;
		free(pathc);
		return -1;
	}
	OBEX_CharToUnicode(ucname, name, ucname_len);

	BTDEBUG("Sending file: %s, path: %s, size: %d\n", name, path, file_size);

	// create new object
	oo = OBEX_ObjectNew(clt->handle, OBEX_CMD_PUT);
	hv.bq4 = clt->con_id;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_CONNECTION, hv, 4, 0);
	hv.bs = ucname;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_NAME, hv, ucname_len, 0);
	hv.bq4 = file_size;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_LENGTH, hv, sizeof(guint32), 0);
	hv.bs = NULL;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_BODY, hv, 0, OBEX_FL_STREAM_START);

	err = OBEX_Request(clt->handle, oo);
	if (!err)
		err = handle_response(clt, BT_SERVICE);

	g_free(ucname);
	g_free(clt->buf); clt->buf = NULL;
	close(clt->sfd); clt->sfd = -1;
	free(pathc);

	return err;
}

int obex_remove(obexclt_t *clt, char *name)
{
	int			err;
	obex_object_t		*oo;	// OBEX Object
	obex_headerdata_t	hv;
	gchar 			*ucname;
	int			ucname_len;

	clt->opcode = OBEX_REMOVE;

	ucname_len = (strlen(name)+1)<<1;
	ucname = g_malloc(ucname_len);
	if (!ucname) {
		return -1;
	}
	OBEX_CharToUnicode(ucname, name, ucname_len);

	BTDEBUG("Removing file: %s\n", name);

	// create new object
	oo = OBEX_ObjectNew(clt->handle, OBEX_CMD_PUT);
	hv.bq4 = clt->con_id;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_CONNECTION, hv, 4, 0);
	hv.bs = ucname;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_NAME, hv, ucname_len, 0);
	
	err = OBEX_Request(clt->handle, oo);
	if (!err)
		err = handle_response(clt, BT_SERVICE);

	g_free(ucname);

	return err;
}

		
int obex_get(obexclt_t *clt, char *name)
{
	int			err;
	obex_object_t		*oo;	// OBEX Object
	obex_headerdata_t	hv;
	gchar 			*ucname;
	int			ucname_len;

	clt->opcode = OBEX_GET;
	clt->buf = name;

	clt->sfd = open(name, O_RDWR | O_CREAT | O_TRUNC, DEFFILEMODE);
	if (clt->sfd < 0) {
		BTDEBUG("Unable to create file: %s\n", name);
		return -1;
	}

	ucname_len = (strlen(name)+1)<<1;
	ucname = g_malloc(ucname_len);
	if (!ucname){
		close(clt->sfd); clt->sfd = -1;
		return -1;
	}
	OBEX_CharToUnicode(ucname, name, ucname_len);

	// create new object
	oo = OBEX_ObjectNew(clt->handle, OBEX_CMD_GET);
	hv.bq4 = clt->con_id;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_CONNECTION, hv, 4, 0);
	hv.bs = ucname;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_NAME, hv, ucname_len, 0);

	OBEX_ObjectReadStream(clt->handle, oo, NULL);	//XXX: Enable streamming

	err = OBEX_Request(clt->handle, oo);
	if (!err)
		err = handle_response(clt, BT_SERVICE);
	
	g_free(ucname);
	close(clt->sfd); clt->sfd = -1;

	return err;
}

int __obex_setpath(obexclt_t *clt, char *path, int flags)
{
	int			err;
	obex_object_t		*oo;	// OBEX Object
	obex_headerdata_t	hv;
	gchar 			*ucname = NULL;
	int			ucname_len = 0, is_name;
	obex_setpath_hdr_t	nonhdrdata;

	clt->opcode = OBEX_SETPATH;
	nonhdrdata.flags = 0x02;
	nonhdrdata.constants = 0;

	if (path == NULL || *path == '\0') {
		/* empty name. set to root */
		ucname = "";
		ucname_len = 0;
		is_name = 1;
	} else if (strcmp(path, "..") != 0) {
		ucname_len = (strlen(path)+1)<<1;
		ucname = g_malloc(ucname_len);
		if (!ucname){
			return -1;
		}
		OBEX_CharToUnicode(ucname, path, ucname_len);
		is_name = 2;
	} else {
		nonhdrdata.flags = 0x03;
		is_name = 0;
	}

	if (flags & SETPATH_CREATE) {
		if (is_name < 2)
			return -EINVAL;
		nonhdrdata.flags &= ~0x02;
	}

	// create new object
	oo = OBEX_ObjectNew(clt->handle, OBEX_CMD_SETPATH);
	OBEX_ObjectSetNonHdrData(oo, (guint8*)&nonhdrdata, 2);
	hv.bq4 = clt->con_id;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_CONNECTION, hv, 4, 0);

	if (is_name) {
		BTDEBUG("Setting name: %d\n", ucname_len);
		hv.bs = ucname;
		OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_NAME, hv, ucname_len, 0);
	}
	
	err = OBEX_Request(clt->handle, oo);
	if (!err)
		err = handle_response(clt, BT_SERVICE);

	if (is_name == 2)
		g_free(ucname);

	return err;
}


obexclt_t *obex_connect(void *addr, int *err)
{
	obexclt_t	*clt;

#if defined(CONFIG_AFFIX_SDP)
	ServiceID = ObexFileTransferServiceClassID;
#endif
	target = browsing;
	target_len = 16;

	clt = __obex_connect(addr, err);
	if (clt == NULL) {
		BTDEBUG("unable to connect to the server\n");
		return NULL;
	}

	return clt;

}

int obex_push(void *addr, char *path)
{
	int			err;
	obex_object_t		*oo;	// OBEX Object
	obex_headerdata_t	hv;
	int			file_size;
	gchar 			*ucname;
	int			ucname_len;
	char			*name, *pathc;
	obexclt_t		*clt;

#if defined(CONFIG_AFFIX_SDP)
	ServiceID = ObexObjectPushServiceClassID;
#endif
	clt = __obex_connect(addr, &err);
	if (clt == NULL) {
		BTDEBUG("unable to connect to the server\n");
		return err;
	}
	clt->opcode = OBEX_PUSH;

	/* set path */
	pathc = strdup(path);
	name = basename(pathc);

	clt->sfd = open(path, O_RDONLY, 0);
	if (clt->sfd < 0) {
		BTDEBUG("File not found: %s\n", pathc);
		free(pathc);
		obex_disconnect(clt);
		return -1;
	}

	file_size = get_filesize(path);

	clt->buf = g_malloc(OBEX_STREAM_CHUNK);
	if (!clt->buf) {
		close(clt->sfd);
		obex_disconnect(clt);
		free(pathc);
		return -1;
	}
		
	ucname_len = (strlen(name)+1)<<1;
	ucname = g_malloc(ucname_len);
	if (!ucname) {
		g_free(clt->buf);
		close(clt->sfd);
		obex_disconnect(clt);
		free(pathc);
		return -1;
	}
	OBEX_CharToUnicode(ucname, name, ucname_len);

	BTDEBUG("Sending file: %s, path: %s, size: %d\n", name, path, file_size);

	// create new object
	oo = OBEX_ObjectNew(clt->handle, OBEX_CMD_PUT);
	hv.bs = ucname;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_NAME, hv, ucname_len, 0);
	hv.bq4 = file_size;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_LENGTH, hv, sizeof(guint32), 0);
	hv.bs = NULL;
	OBEX_ObjectAddHeader(clt->handle, oo, OBEX_HDR_BODY, hv, 0, OBEX_FL_STREAM_START);
	
	err = OBEX_Request(clt->handle, oo);
	if (!err)
		err = handle_response(clt, BT_SERVICE);

	g_free(ucname);
	g_free(clt->buf);
	close(clt->sfd);
	obex_disconnect(clt);
	free(pathc);

	return err;
}


