/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   Based on the from Pontus Fuchs <pontus.fuchs@tactel.se>
   Copyright (c) 1999, 2000 Pontus Fuchs, All Rights Reserved.
   
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
   $Id: obex_server.h,v 1.1.1.1 2002/07/11 11:21:58 kds Exp $

   OBEX server header file for Affix

   Adapted for Affix, Fixes: 
   				Imre Deak <ext-imre.deak@nokia.com>
   				Dmitry Kasatkin
*/

#ifndef OBEX_SERVER_H
#define OBEX_SERVER_H

#include <termios.h>

/* Affix includes*/
#include <affix/obex.h>


#define CHDIR_CHECK_ONLY		0x0001
#define CHDIR_CREATE_IF_NOT_EXIST	0x0002
#define CHDIR_CREATE_RECURSIVE		0x0004

typedef struct obex_common_hdr {
	guint8  opcode;
	guint16 len;
} __PACK__ obex_common_hdr_t;

typedef struct obex_connect_hdr {
	guint8  version;
	guint8  flags;
	guint16 mtu;
} __PACK__ obex_connect_hdr_t;


typedef struct obex_setpath_hdr {
	guint8  flags;
	guint8	constants;
} __PACK__ obex_setpath_hdr_t;


enum obexsrv_state {
	SRVSTATE_IDLE = 0,
	SRVSTATE_CLOSED
};

typedef struct  {
	obex_link_t	link;
	obex_t		*handle;
	int		state;
        gboolean	serverdone;
	gboolean	streamming;
        char		*name;
	int		sfd;
	char		*buf;
} obexsrv_t;

int change_dir(char *new_dir, int mode);

void server_start(obexsrv_t *srv, char *service);
void server_done(obex_t *handle, obex_object_t *object, gint obex_cmd, gint obex_rsp);
void server_request(obex_t *handle, obex_object_t *object, gint cmd);

obex_t *btobex_init(obex_link_t *link, obex_event_t eventcb, guint flags);
void btobex_cleanup(obex_link_t *link, gboolean force);

obex_object_t *build_object_from_file(obex_t *handle, const char *filename);
gint safe_save_file(gchar *name, const guint8 *buf, gint len);
guint8* easy_readfile(const char *filename, int *file_size);

#endif

