/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author:Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   */

/*
   $Id: attr.c,v 1.2 2002/07/19 15:18:34 kds Exp $

Fixes:     Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>
*/

#include <affix/config.h>

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdint.h>
#include <netinet/in.h>

#include <affix/sdp.h>
#include <affix/sdpclt.h>
#include "attr.h"


void printPduForm(PDUForm *pduForm)
{
	if (pduForm->dataSize == 0)
		BTDEBUG("NULL buffer");
	BTDUMP(pduForm->data, pduForm->dataSize);
}

int isSequenceData(SDPData *pSDPData)
{
	if ((pSDPData->dtd == SequenceSize8) || 
			(pSDPData->dtd == SequenceSize16) ||
			(pSDPData->dtd == SequenceSize32) ||
			(pSDPData->dtd == SequenceAlternatesSize8) || 
			(pSDPData->dtd == SequenceAlternatesSize32) ||
			(pSDPData->dtd == SequenceAlternatesSize16)) {
		return 1;
	}
	else
		return 0;
}

int verifySDPData(SDPData *pSDPData)
{
	return 1;
}


SDPData *createSDPData(DataTypeDescriptor dtd, char *pValue)
{
	SDPData *pSeqEntry = NULL;
	SDPData *pSDPData = NULL;
	int strLength = 0;

	if (pValue != NULL) {
		pSDPData = (SDPData *)malloc(sizeof(SDPData));
		memset(pSDPData, 0, sizeof(SDPData));
		pSDPData->dtd = dtd;
		pSDPData->unitSize = sizeof(DataTypeDescriptor);

		switch (dtd) {
			case UnsignedInteger_8 :
				pSDPData->sdpDataValue.uint8 = *(uint8_t *)pValue;
				pSDPData->unitSize += sizeof(uint8_t);
				break;
			case Integer_8 :
			case Boolean :
				pSDPData->sdpDataValue.int8 = *(int8_t *)pValue;
				pSDPData->unitSize += sizeof(int8_t);
				break;
			case UnsignedInteger_16 :
				pSDPData->sdpDataValue.uint16 = *(uint16_t *)pValue;
				pSDPData->unitSize += sizeof(uint16_t);
				break;
			case Integer_16 :
				pSDPData->sdpDataValue.int16 = *(int16_t *)pValue;
				pSDPData->unitSize += sizeof(int16_t);
				break;
			case UnsignedInteger_32 :
				pSDPData->sdpDataValue.uint32 = *(uint32_t *)pValue;
				pSDPData->unitSize += sizeof(uint32_t);
				break;
			case Integer_32 :
				pSDPData->sdpDataValue.int32 = *(int32_t *)pValue;
				pSDPData->unitSize += sizeof(int32_t);
				break;
			case Integer_64 :
				pSDPData->sdpDataValue.int64 = *(int64_t *)pValue;
				pSDPData->unitSize += sizeof(int64_t);
				break;
			case UnsignedInteger_64 :
				pSDPData->sdpDataValue.uint64 = *(uint64_t *)pValue;
				pSDPData->unitSize += sizeof(uint64_t);
				break;
			case UnsignedInteger_128 :
				memcpy(&pSDPData->sdpDataValue.uint128, pValue, sizeof(Int128Bit));
				pSDPData->unitSize += sizeof(int64_t);
				break;
			case Integer_128 :        
				memcpy(&pSDPData->sdpDataValue.int128, pValue, sizeof(Int128Bit));
				pSDPData->unitSize += sizeof(uint64_t);
				break;
			case UUID_16 :
				makeUUIDFrom16Bits(&pSDPData->sdpDataValue.uuid, *(UUID16Bit *)pValue);
				pSDPData->unitSize += sizeof(UUID16Bit);
				break;
			case UUID_32 :
				makeUUIDFrom32Bits(&pSDPData->sdpDataValue.uuid, *(UUID32Bit *)pValue);
				pSDPData->unitSize += sizeof(UUID32Bit);
				break;
			case UUID_128 :
				makeUUIDFrom128Bits(&pSDPData->sdpDataValue.uuid, (UUID128Bit *)pValue);
				pSDPData->unitSize += sizeof(UUID128Bit);
				break;
			case URLStringSize8 :
			case TextStringSize8 :
			case URLStringSize16 :
			case TextStringSize16 :
				strLength = strlen(pValue);
				pSDPData->unitSize += strLength;
				if (strLength <= USHRT_MAX) {
					pSDPData->sdpDataValue.stringPtr = (char *)malloc(strLength+1);
					strcpy(pSDPData->sdpDataValue.stringPtr, pValue);
					if (strLength <= UCHAR_MAX) {
						pSDPData->unitSize += sizeof(uint8_t);
						if ((dtd != URLStringSize8) && (dtd != TextStringSize8)) {
							if (dtd == URLStringSize16)
								dtd = URLStringSize8;
							else
								dtd = TextStringSize8;
						}
					}
					else {
						pSDPData->unitSize += sizeof(uint16_t);
						if ((dtd != URLStringSize16) && (dtd != TextStringSize16)) {
							if (dtd == URLStringSize8)
								dtd = URLStringSize16;
							else
								dtd = TextStringSize16;
						}
					}
				}
				else
					BTERROR("Strings of size > USHRT_MAX not supported\n");
				break;
			case URLStringSize32 :
			case TextStringSize32 :
				BTERROR("Strings of size > USHRT_MAX not supported\n");
				break;
			case SequenceAlternatesSize8 :
			case SequenceAlternatesSize16 :
			case SequenceAlternatesSize32 :
			case SequenceSize8 :
			case SequenceSize16 :
			case SequenceSize32 :
				pSeqEntry = (SDPData *)pValue;
				pSDPData->sdpDataValue.dataSeq = pSeqEntry;
				while (pSeqEntry != NULL) {
					pSDPData->unitSize += pSeqEntry->unitSize;
					pSeqEntry = pSeqEntry->next;
				}
				break;
			default :
				free(pSDPData); pSDPData = NULL;
				break;
		}
	}

	return pSDPData;
}


void addSDPDataToSequence(SDPData *pSequence, SDPData *pSDPData)
{
	SDPData *elementToAppendTo = NULL;

	if ((pSequence != NULL) && (pSDPData != NULL)) {
		elementToAppendTo = pSequence;
		pSDPData->next = NULL;
		while (elementToAppendTo->next != NULL) {
			elementToAppendTo = elementToAppendTo->next;
		}
		elementToAppendTo->next = pSDPData;
	}
}


SDPData *createSDPDataSequence(GPtrArray *pDataTypeDescPointerArray, GPtrArray *pDataValuePointerArray)
{
	SDPData *pSDPDataSequence = NULL;
	SDPData *currentPtr, *seqStartPtr;
	int status, i;

	short typeEntries = pDataTypeDescPointerArray->len;
	short valueEntries = pDataValuePointerArray->len;

	if (typeEntries == valueEntries) {
		currentPtr = NULL; seqStartPtr = NULL; status = 0;
		for (i = 0; i < typeEntries; i++) {
			SDPData *pSDPDataEntry = NULL;

			DataTypeDescriptor dtd = *(DataTypeDescriptor *)
				g_ptr_array_index(pDataTypeDescPointerArray, i);
			char *value = 
				(char *)g_ptr_array_index(pDataValuePointerArray, i);

			if ((dtd >= SequenceSize8) && (dtd <= SequenceAlternatesSize32))
				pSDPDataEntry  = (SDPData *)value;
			else
				pSDPDataEntry = createSDPData(dtd, value);

			if (pSDPDataEntry != NULL) {
				if (currentPtr == NULL) {
					currentPtr = pSDPDataEntry;
					seqStartPtr = currentPtr;
				}
				else {
					currentPtr->next = pSDPDataEntry;
					currentPtr = pSDPDataEntry;
				}
			}
			else {
				free(pSDPDataSequence);
				pSDPDataSequence = NULL;
				status = -1;
				break;
			}
		}
		if (status == 0)
			pSDPDataSequence = createSDPData(SequenceSize8, (char *)seqStartPtr);
	}

	return pSDPDataSequence;
}

int addSDPAlternateData(SDPData *pOriginalValue, SDPData *pAlternateValue)
{
	int status = -1;

	if ((pOriginalValue != NULL) && (pAlternateValue != NULL)) {
		if (pOriginalValue->alternates == NULL) {
			pOriginalValue->alternates = (SDPData **)malloc(sizeof(SDPData *) * 5);
			pOriginalValue->alternatesCount = 0;
		}
		else
			if ((pOriginalValue->alternatesCount % 5) == 0) {
				pOriginalValue->alternates = (SDPData **)realloc(
						pOriginalValue->alternates, ((5 * sizeof(SDPData *)) *
							(1 + pOriginalValue->alternatesCount / 5)));
			}
		pOriginalValue->alternates[pOriginalValue->alternatesCount] = 
			pAlternateValue;
		pOriginalValue->alternatesCount++;

		/*
		 ** Set the DTD
		 */
		status = 0;
		pOriginalValue->unitSize += pAlternateValue->unitSize;
		if (pOriginalValue->unitSize < UCHAR_MAX) {
			pOriginalValue->dtd = SequenceAlternatesSize8;
		}
		else
			if (pOriginalValue->unitSize < USHRT_MAX) {
				pOriginalValue->dtd = SequenceAlternatesSize16;
			}
			else
				pOriginalValue->dtd = SequenceAlternatesSize32;
	}
	else
		BTDEBUG("AddSDPData : Original value is : 0x%x, 
				Alternate value is : 0x%x,
				but either of them cannot be NULL\n", 
				(uint32_t)pOriginalValue, (uint32_t)pAlternateValue);

	return status;
}

int createSDPAttribute( ServiceRecord *svcRec,
		ServiceAttributeIdentifier attrId, SDPData *pSDPData)
{
	int status = -1;

	if ((svcRec != NULL) && (pSDPData != NULL)) {
		if (verifySDPData(pSDPData)) {
			pSDPData->attrId = attrId;
			addToAttributeList(svcRec, attrId, pSDPData, 0);
			status = 0;
		}
	}

	return status;
}


void removeSDPAttribute(ServiceRecord *svcRec, ServiceAttributeIdentifier attrId)
{
	SDPData *pSDPData = NULL;

	if ((svcRec != NULL) && (svcRec->attributeList != NULL)) {
		pSDPData = getSDPData(svcRec, attrId);
		if (pSDPData != NULL)
			svcRec->attributeList = g_slist_remove(svcRec->attributeList, pSDPData);
	}
}


void setSequenceLength(char *sequenceStartPtr, int length)
{
	DataTypeDescriptor dtd;

	dtd = __get_u8(sequenceStartPtr);
	switch (dtd) {
		case SequenceSize8 :
		case SequenceAlternatesSize8 :
		case TextStringSize8 :
		case URLStringSize8 :
			__put_u8(sequenceStartPtr + 1, length);
			break;

		case SequenceSize16 :
		case SequenceAlternatesSize16 :
		case TextStringSize16 :
		case URLStringSize16 :
			__put_u16(sequenceStartPtr + 1, htons(length));
			break;

		case SequenceSize32 :
		case SequenceAlternatesSize32 :
		case TextStringSize32 :
		case URLStringSize32 :
			__put_u32(sequenceStartPtr + 1, htonl(length));
			break;
	}
}

int setDataTypeDescriptor(PDUForm *pduForm, DataTypeDescriptor dtd)
{
	char *pdata;
	int dataSize = 0;
	int originalSize = 0;

	if (pduForm != NULL) {
		originalSize = pduForm->dataSize;
		pdata = pduForm->data + pduForm->dataSize;
		__put_u8(pdata, dtd);
		pdata += sizeof(DataTypeDescriptor);
		pduForm->dataSize += sizeof(DataTypeDescriptor);

		switch (dtd) {
			case SequenceSize8 :
			case TextStringSize8 :
			case URLStringSize8 :
			case SequenceAlternatesSize8 :
				pduForm->dataSize += sizeof(uint8_t);
				break;
			case SequenceSize16 :
			case TextStringSize16 :
			case URLStringSize16 :
			case SequenceAlternatesSize16 :
				pduForm->dataSize += sizeof(uint16_t);
				break;
			case SequenceSize32 :
			case TextStringSize32 :
			case URLStringSize32 :
			case SequenceAlternatesSize32 :
				pduForm->dataSize += sizeof(uint32_t);
				break;
		}

		dataSize = pduForm->dataSize - originalSize;
	}

	return dataSize;
}


void setAttributeId(PDUForm *pduForm, ServiceAttributeIdentifier svcAttrId)
{
	char *pdata;

	if (pduForm != NULL) {
		pdata = pduForm->data;
		/*
		 ** Set the attribute ID 
		 */

		/* Data type for SvcAttrId */
		__put_u8(pdata, UnsignedInteger_16);
		pdata += sizeof(DataTypeDescriptor);
		pduForm->dataSize = sizeof(DataTypeDescriptor);

		__put_u16(pdata, htons(svcAttrId));
		pdata += sizeof(ServiceAttributeIdentifier);
		pduForm->dataSize += sizeof(ServiceAttributeIdentifier);
	}
}


int generateDataSeqPDUForm(PDUForm *dstPduForm,	SDPData *pSDPData)
{
	SDPData *localSDPData;
	int dataSize = 0;

	localSDPData = pSDPData->sdpDataValue.dataSeq;
	if (localSDPData != NULL) {
		do {
			dataSize += generatePDUForm(dstPduForm, localSDPData);
#ifdef CONFIG_AFFIX_DEBUG
			printPduForm(dstPduForm);
#endif
			localSDPData = localSDPData->next;
		} while (localSDPData != NULL);
	}

	return dataSize;
}

int generatePDUForm(PDUForm *dstPduForm, SDPData *pSDPData)
{
	int pduSize = 0;
	int dataSize = 0;
	int dataAlternatesSize = 0;
	unsigned char *pSrc = NULL;
	unsigned char sequenceType = 0;
	unsigned char sequenceAlternatesType = 0;
	DataTypeDescriptor dtd;
	DataTypeDescriptor savedDTD;
	char *sequenceStartPtr = NULL;
	uint16_t uint16nbo;
	uint32_t uint32nbo;
	int i;

	dtd = pSDPData->dtd;

	if (pSDPData != NULL) {

		sequenceStartPtr = dstPduForm->data + dstPduForm->dataSize;
		pduSize = setDataTypeDescriptor(dstPduForm, dtd);
		switch (dtd) {
			case UnsignedInteger_8 :
				pSrc = &pSDPData->sdpDataValue.uint8;
				dataSize = sizeof(uint8_t);
				break;
			case UnsignedInteger_16 :
				uint16nbo = htons(pSDPData->sdpDataValue.uint16);
				pSrc = (unsigned char *)&uint16nbo;
				dataSize = sizeof(uint16_t);
				break;
			case UnsignedInteger_32 :
				uint32nbo = htonl(pSDPData->sdpDataValue.uint32);
				pSrc = (unsigned char *)&uint32nbo;
				dataSize = sizeof(uint32_t);
				break;
			case UnsignedInteger_64 :
				pSrc = (unsigned char *)&pSDPData->sdpDataValue.uint64;
				dataSize = sizeof(uint64_t);
				break;
			case UnsignedInteger_128 :
				pSrc = (unsigned char *)&pSDPData->sdpDataValue.uint128;
				dataSize = sizeof(Int128Bit);
				break;
			case Integer_8 :
			case Boolean :
				pSrc = (unsigned char *)&pSDPData->sdpDataValue.int8;
				dataSize = sizeof(int8_t);
				break;
			case Integer_16 :
				uint16nbo = htons(pSDPData->sdpDataValue.int16);
				pSrc = (unsigned char *)&uint16nbo;
				dataSize = sizeof(int16_t);
				break;
			case Integer_32 :
				uint32nbo = htonl(pSDPData->sdpDataValue.int32);
				pSrc = (unsigned char *)&uint32nbo;
				dataSize = sizeof(int32_t);
				break;
			case Integer_64 :
				pSrc = (unsigned char *)&pSDPData->sdpDataValue.int64;
				dataSize = sizeof(int64_t);
				break;
			case Integer_128 :
				pSrc = (unsigned char *)&pSDPData->sdpDataValue.int128;
				dataSize = sizeof(Int128Bit);
				break;

			case TextStringSize8 :
			case URLStringSize8 :
			case TextStringSize16 :
			case TextStringSize32 :
			case URLStringSize16 :
			case URLStringSize32 :
				pSrc = (unsigned char *)pSDPData->sdpDataValue.stringPtr;
				dataSize = strlen(pSDPData->sdpDataValue.stringPtr);
				setSequenceLength(sequenceStartPtr, dataSize);
				break;

			case SequenceSize8 :
			case SequenceSize16 :
			case SequenceSize32 :
				sequenceType = 1;
				dataSize = 0;
				dataSize = generateDataSeqPDUForm(dstPduForm, pSDPData);
				setSequenceLength(sequenceStartPtr, dataSize);
				break;

			case SequenceAlternatesSize8:
			case SequenceAlternatesSize16 :
			case SequenceAlternatesSize32 :
				sequenceAlternatesType = 1;
				savedDTD = pSDPData->dtd;
				pSDPData->dtd = SequenceSize8;
				dataSize = generatePDUForm(dstPduForm, pSDPData);
				pSDPData->dtd = savedDTD;
				dataAlternatesSize = 0;
				for (i = 0; i < pSDPData->alternatesCount; i++) {
					dataAlternatesSize += generatePDUForm(dstPduForm, 
							pSDPData->alternates[i]);
				}
				dataSize += dataAlternatesSize;
				setSequenceLength(sequenceStartPtr, dataSize);
				break;

			case UUID_16 :
				uint16nbo = htons(pSDPData->sdpDataValue.uuid.value.uuid16Bit);
				pSrc = (unsigned char *)&uint16nbo;
				dataSize = sizeof(UUID16Bit);
				break;

			case UUID_32 :
				uint32nbo = htonl(pSDPData->sdpDataValue.uuid.value.uuid32Bit);
				pSrc = (unsigned char *)&uint32nbo;
				dataSize = sizeof(UUID32Bit);
				break;

			case UUID_128 :
				pSrc = (unsigned char *)&pSDPData->sdpDataValue.uuid.value.uuid128Bit;
				dataSize = sizeof(UUID128Bit);
				break;

			default :
				break;
		}

		if ((sequenceType == 0) && (sequenceAlternatesType == 0)) {
			if ((pSrc != NULL) && (dstPduForm != NULL)) {
				memcpy((dstPduForm->data + dstPduForm->dataSize), 
						pSrc, dataSize);
				dstPduForm->dataSize += dataSize;
			}
			else
				BTDEBUG("Gen PDU : Cant copy from NULL source or dest\n");
		}
		pduSize += dataSize;
	}

	return pduSize;
}


void attrPduForm(gpointer value, gpointer userData)
{
	SDPData *pSDPData;

	if (value != NULL) {
		DataTypeDescriptor dtd;
		int pduFormSize = 0;
		pSDPData = (SDPData *)value;

		dtd = pSDPData->dtd;

		if (dtd < UUID_128)
			pduFormSize = BasicAttributePduFormSize;
		else
			pduFormSize = SequencePduFormSize;

		if (pSDPData->pduForm.data == NULL) {
			pSDPData->pduForm.data = (char *)malloc(pduFormSize);
			pSDPData->pduForm.bufferSize = pduFormSize;
		}
		pSDPData->pduForm.dataSize = 0;
		memset(pSDPData->pduForm.data, 0, pSDPData->pduForm.bufferSize);

#ifdef CONFIG_AFFIX_DEBUG
		BTDEBUG("[%s-%d] : ", __FUNCTION__, __LINE__);
		printPduForm(&pSDPData->pduForm);
		BTDEBUG("Adding attr id : 0x%x", pSDPData->attrId);
#endif
		setAttributeId(&pSDPData->pduForm, pSDPData->attrId);
		generatePDUForm(&pSDPData->pduForm, pSDPData);
#ifdef CONFIG_AFFIX_DEBUG
		printPduForm(&pSDPData->pduForm);
#endif

		/*
		 ** Append to the overall PDU form
		 */
		if (userData != NULL)
			appendToPduForm((PDUForm *)userData, &pSDPData->pduForm);
	}
}

void generateServiceRecordPduForm(ServiceRecord *svcRec)
{
	GSList *svcRecList = NULL;

	if (svcRec != NULL) {
		svcRecList = svcRec->attributeList;
		if (svcRecList != NULL) {
			if (svcRec->pduForm.data == NULL) {
				svcRec->pduForm.data = (char *)malloc(SDP_PDU_CHUNK_SIZE);
				svcRec->pduForm.bufferSize = SDP_PDU_CHUNK_SIZE;
			}
			svcRec->pduForm.dataSize = 0;
			memset(svcRec->pduForm.data, 0, svcRec->pduForm.bufferSize);
			g_slist_foreach(svcRecList, attrPduForm, &svcRec->pduForm);
			//setSequenceLengthPDU(&svcRec->pduForm);
#ifdef CONFIG_AFFIX_DEBUG
			BTDEBUG("[%s-%d] : ", __FUNCTION__, __LINE__);
			printPduForm(&svcRec->pduForm);
#endif
		}
	}
}


int addToAttributeList(
		ServiceRecord *svcRec,
		ServiceAttributeIdentifier attrId,
		SDPData *pSDPData,
		int replace
		)
{
	int status = 0;
	SDPData *pData = NULL;

	BTDEBUG("Adding attr id : 0x%x to attr table", attrId);

	if (svcRec->attributeList == NULL) {
		BTDEBUG("Creating a new attr list");
	}
	BTDEBUG("Attr list size : %d", g_slist_length(svcRec->attributeList));

	pData = getSDPData(svcRec, attrId);
	if ((replace) && (pData != NULL)) {
		svcRec->attributeList = g_slist_remove(svcRec->attributeList, pData);
		freeSDPData(pData);
		pData = NULL;
	}

	if (pData == NULL) {
		pSDPData->attrId = attrId;
		svcRec->attributeList = g_slist_insert_sorted(svcRec->attributeList, 
				(gpointer)pSDPData, attrIdCompareFunc);
	}
	else {
		BTDEBUG("Attribute with id : 0x%x already exists in table : 0x%x\n",
				attrId, (uint32_t)svcRec);
		status = -1;
	}


	return status;
}

