/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author:Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   */

/*
   $Id: des.c,v 1.2 2002/07/19 15:18:34 kds Exp $

Fixes:     Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>
Manel Guerrero Zapata <manel.guerrero-zapata@nokia.com>
*/

#include <affix/config.h>

#include <glib.h>
#include <malloc.h>
#include <string.h>
#include <stdio.h>

#include <affix/sdp.h>
#include <affix/sdpclt.h>
#include "utils.h"
#include "servicedb.h"
#include "pdu.h"
#include "attr.h"


SDPData *extractAttribute(
		char *pdata, 
		int *extractedLength, 
		ServiceRecord *svcRec
		);

//void printSDPData(SDPData *);

guint attrIdHashFunc(gconstpointer key)
{
	guint attrId;

	attrId = *(ServiceAttributeIdentifier *)key;
	return attrId;
}

gint attrIdCompareFunc(gconstpointer key1, gconstpointer key2)
{
	gint cmp = 0;
	SDPData *pData1 = NULL;
	SDPData *pData2 = NULL;

	pData1 = (SDPData *)key1; pData2 = (SDPData *)key2;
	if ((pData1 != NULL) && (pData2 != NULL)) {
		cmp = pData1->attrId - pData2->attrId;
	}

	return cmp;
}

void freeSDPData(SDPData *pSDPData)
{
}

void freeDataSeq(SDPData *pDataSeq)
{
	SDPData *currentNode = NULL;

	/*
	 ** Cleanup our list (if any)
	 */
	currentNode = pDataSeq->sdpDataValue.dataSeq;
	while (currentNode != NULL) {
		freeSDPData(currentNode);
		currentNode = currentNode->next;
	}
	free(pDataSeq);
}


SDPData *extractInteger(char *pdata, int *extractedLength)
{
	SDPData *pIntData = NULL;
	DataTypeDescriptor dtd;

	BTDEBUG("Extracting integer");
	dtd = __get_u8(pdata);
	pdata += sizeof(DataTypeDescriptor);
	*extractedLength += sizeof(DataTypeDescriptor);

	pIntData = (SDPData *)malloc(sizeof(SDPData));
	memset(pIntData, 0, sizeof(SDPData));
	switch (dtd) {
		case Boolean :
		case Integer_8 :
		case UnsignedInteger_8 :
			*extractedLength += sizeof(uint8_t);
			pIntData->sdpDataValue.uint8 = __get_u8(pdata);
			break;
		case Integer_16 :
		case UnsignedInteger_16 :
			*extractedLength += sizeof(uint16_t);
			pIntData->sdpDataValue.uint16 = ntohs(__get_u16(pdata));
			break;
		case Integer_32 :
		case UnsignedInteger_32 :
			*extractedLength += sizeof(uint32_t);
			pIntData->sdpDataValue.uint32 = ntohl(__get_u32(pdata));
			break;
			/*
			 ** Not clear if NBO rules apply here and if yes, how
			 */
		case Integer_64 :
		case UnsignedInteger_64 :
			*extractedLength += sizeof(uint64_t);
			break;
		case Integer_128 :
		case UnsignedInteger_128 :
			*extractedLength += sizeof(Int128Bit);
			break;
		default :
			free(pIntData);
			pIntData = NULL;
	}

#ifdef CONFIG_AFFIX_DEBUG
	if (pIntData != NULL)
		BTDEBUG("Ingeter : 0x%x", pIntData->sdpDataValue.uint32);
#endif

	return pIntData;
}

SDPData *extractUUIDRec(char *pdata, int *extractedLength, ServiceRecord *svcRec)
{
	int status = 0;
	SDPData *pUUIDData = NULL;

	BTDEBUG("Extracting UUID");
	pUUIDData = (SDPData *)malloc(sizeof(SDPData));
	memset(pUUIDData, 0, sizeof(SDPData));
	status = extractUUID(pdata, &pUUIDData->sdpDataValue.uuid, extractedLength);
	if (status == -1) {
		free(pUUIDData); pUUIDData = NULL;
	}
	else
		addUUIDToTargetPattern(svcRec, &pUUIDData->sdpDataValue.uuid);

	return pUUIDData;
}

/*
 ** Extract strings from the PDU (could be service description
 ** and similar info) 
 */
int extractStringInt(char *buffer, char **dstStringBuffer, int *bytesScanned)
{
	char *pdata;
	int status;
	int stringLength;
	DataTypeDescriptor dataType;

	pdata = buffer; status = 0; stringLength = 0;
	dataType = __get_u8(pdata);
	pdata += sizeof(DataTypeDescriptor);
	*bytesScanned += sizeof(DataTypeDescriptor);

	switch (dataType) {
		case TextStringSize8 :
		case URLStringSize8 :
			stringLength = __get_u8(pdata);
			pdata += sizeof(DataSizeDescriptor8);
			*bytesScanned += sizeof(DataSizeDescriptor8) + stringLength;
			break;

		case TextStringSize16 :
		case URLStringSize16 :
			stringLength = ntohs(__get_u16(pdata));
			pdata += sizeof(DataSizeDescriptor16);
			*bytesScanned += sizeof(DataSizeDescriptor16) + stringLength;
			break;

		default :
			BTERROR("Sizeof text string > UINT16_MAX");
			status = -1;
			break;
	}

	if (status != -1) {
		/*
		 ** Extract the string
		 */
		*dstStringBuffer = (char *)malloc(stringLength + 1);
		memset(*dstStringBuffer, 0, (stringLength + 1));
		strncpy(*dstStringBuffer, pdata, stringLength);
		BTDEBUG("Len : %d", stringLength);
		BTDEBUG("Str : %s", *dstStringBuffer);
	}

	return status;
}

SDPData *extractString(char *pdata, int *extractedLength)
{
	int status = 0;
	SDPData *pStringData = NULL;

	BTDEBUG("Extracting string");
	pStringData = (SDPData *)malloc(sizeof(SDPData));
	memset(pStringData, 0, sizeof(SDPData));
	status = extractStringInt(pdata, &pStringData->sdpDataValue.stringPtr, 
			extractedLength);
	if (status == -1) {
		free(pStringData); pStringData = NULL;
	}

	return pStringData;
}

SDPData *extractSequence(char *pdata, int *extractedLength, ServiceRecord *svcRec)
{
	int sequenceLength = 0;
	int localExtractedLength = 0;
	SDPData *currentNode, *previousNode;
	int seqElementCount = 0;
	char *localSeqPtr = NULL;
	DataTypeDescriptor dtd;
	SDPData *pDataSeq = NULL;
	int attributeLength = 0;

	BTDEBUG("Extracting sequence ");
	localSeqPtr = extractSequenceType(pdata, &dtd, &sequenceLength, 
			extractedLength);
	BTDEBUG("Sequence Type : 0x%x length : 0x%x", dtd, sequenceLength);

	if ((localSeqPtr != NULL) && (sequenceLength != 0)) {
		pDataSeq = (SDPData *)malloc(sizeof(SDPData));
		memset(pDataSeq, 0, sizeof(SDPData));
		currentNode = NULL; previousNode = NULL;
		while (localExtractedLength < sequenceLength) {
			attributeLength = 0;
			currentNode = extractAttribute(localSeqPtr, &attributeLength, svcRec);
			if (currentNode != NULL) {
				seqElementCount++;
				if (previousNode != NULL)
					previousNode->next = currentNode;
				else {
					/*
					 ** First element of list
					 */
					pDataSeq->sdpDataValue.dataSeq = currentNode;
				}
				previousNode = currentNode;
				localSeqPtr += attributeLength;
				localExtractedLength += attributeLength;
				BTDEBUG("LocalExtractedLength : %d SequenceLength : %d",
						localExtractedLength, sequenceLength);
			} else
				break;
		}

		if (localExtractedLength != sequenceLength) {
			BTDEBUG("Error in extracting sequence, terminating");
			freeDataSeq(pDataSeq);
			pDataSeq = NULL;
		}
		else
			*extractedLength += localExtractedLength;
	}

	return pDataSeq;
}

SDPData *extractSequenceAlternates(char *pdata, int *extractedLength, ServiceRecord *svcRec)
{
	SDPData *pAltDataSeq = NULL;
	int sequenceLength = 0;
	int localExtractedLength = 0;
	int localAttrLength = 0;
	SDPData *dataSeq;
	char *localSeqPtr = NULL;
	DataTypeDescriptor dtd;

	BTDEBUG("Extracting sequence alternates");
	localSeqPtr = extractSequenceType(pdata, &dtd, &sequenceLength, 
			extractedLength);
	if (localSeqPtr != NULL) {

		int entryCount = 0;
		while (localExtractedLength < sequenceLength) {
			localAttrLength = 0;
			dataSeq = extractAttribute(localSeqPtr, &localAttrLength, svcRec);
			localExtractedLength += localAttrLength;
			localSeqPtr += localAttrLength;
			if (dataSeq != NULL) {
				if (entryCount == 0) {
					pAltDataSeq = dataSeq;
					pAltDataSeq->alternates = (SDPData **)malloc(sizeof(SDPData *));
					pAltDataSeq->alternatesCount = 0;
				}
				else {
					pAltDataSeq->alternates[pAltDataSeq->alternatesCount] = dataSeq;
					pAltDataSeq->alternatesCount++;
				}
				entryCount++;
			}
			else {
				break;
			}
			if ((pAltDataSeq->alternatesCount % 5) == 0) {
				pAltDataSeq->alternates = (SDPData **)realloc(
						pAltDataSeq->alternates, ((5 * sizeof(SDPData *)) *
							(1 + pAltDataSeq->alternatesCount / 5)));
			}
		}

		if (localExtractedLength != sequenceLength) {
			/*
			 ** Cleanup our list (if any)
			 */
			int i = 0;
			while (i < pAltDataSeq->alternatesCount) {
				freeDataSeq(pAltDataSeq->alternates[i]);
				i++;
			}
			free(pAltDataSeq);
			pAltDataSeq = NULL;
		}
		else
			*extractedLength += localExtractedLength;
	}

	return pAltDataSeq;
}


SDPData *extractAttribute(char *pdata, int *extractedLength, ServiceRecord *svcRec)
{
	DataTypeDescriptor dataType;
	SDPData *pDataElem = NULL;
	int localExtractedLength = 0;

	dataType = __get_u8(pdata);

	BTDEBUG("DataTypeDescriptor : 0x%x", dataType);

	switch (dataType) {
		case Boolean :
		case UnsignedInteger_8 :
		case UnsignedInteger_16 :
		case UnsignedInteger_32 :
		case UnsignedInteger_64 :
		case UnsignedInteger_128 :
		case Integer_8 :
		case Integer_16 :
		case Integer_32 :
		case Integer_64 :
		case Integer_128 :
			pDataElem = extractInteger(pdata, &localExtractedLength);
			break;

		case UUID_16 :
		case UUID_32 :
		case UUID_128 :
			pDataElem = extractUUIDRec(pdata, &localExtractedLength, svcRec);
			break;

		case TextStringSize8 :
		case TextStringSize16 :
		case TextStringSize32 :
		case URLStringSize8 :
		case URLStringSize16 :
		case URLStringSize32 :
			pDataElem = extractString(pdata, &localExtractedLength);
			break;

		case SequenceSize8 :
		case SequenceSize16 :
		case SequenceSize32 :
			pDataElem = extractSequence(pdata, &localExtractedLength, svcRec);
			break;

		case SequenceAlternatesSize8 :
		case SequenceAlternatesSize16 :
		case SequenceAlternatesSize32 :
			pDataElem = extractSequenceAlternates(pdata, 
					&localExtractedLength, svcRec);
			break;

		default :
			BTERROR("Unknown data descriptor : 0x%x terminating", dataType);
			free(pDataElem);
			pDataElem = NULL;
			break;
	}

	if (pDataElem != NULL) {
		pDataElem->dtd = dataType;
		*extractedLength += localExtractedLength;
	}

	return pDataElem;
}

int extractAttributes(ServiceRecord *svcRec, char *localSeqPtr, int sequenceLength)
{
	int extractStatus = -1;
	int localExtractedLength = 0;
	DataTypeDescriptor dtd;
	int attrValueLength;// = 0;
	int attrSize;// = 0;
	ServiceAttributeIdentifier attrId;
	SDPData *pAttrValue;// = NULL;

	while (localExtractedLength < sequenceLength) {
		BTDEBUG("Extract PDU, sequenceLength : %d localExtractedLength : %d",
				sequenceLength, localExtractedLength);
		attrSize = 0;
		dtd = __get_u8(localSeqPtr);
		attrSize = sizeof(DataTypeDescriptor);
		attrId = ntohs(__get_u16(localSeqPtr + attrSize));
		attrSize += sizeof(ServiceAttributeIdentifier);
		BTDEBUG("DTD of attrId : %d Attr id : 0x%x ", dtd, attrId);
		attrValueLength = 0;
		pAttrValue = extractAttribute((localSeqPtr+attrSize), &attrValueLength,
				svcRec);
		BTDEBUG("Attr id : 0x%x attrValueLength : %d", attrId, attrValueLength);
		attrSize += attrValueLength;
		if (pAttrValue == NULL) {
			BTDEBUG("[%s-%d] : Terminating extraction of attributes",
					__FUNCTION__, __LINE__);
			break;
		}
		else {
			pAttrValue->attrId = attrId;
			localExtractedLength += attrSize; localSeqPtr += attrSize;

			addToAttributeList(svcRec, attrId, pAttrValue, 1);
			extractStatus = 0;
			BTDEBUG("Extract PDU, sequenceLength : %d localExtractedLength : %d",
					sequenceLength, localExtractedLength);
		}
	}

	if (extractStatus == 0) {
		BTDEBUG("Successful extracting of Svc Rec attributes");
		printServiceAttributes(svcRec->attributeList);
		return sequenceLength;
	}
	
	return 0;
}

ServiceRecord *serverExtractPDU(char *pdata, ServiceRecordHandle handleExpected, int *bytesScanned)
{
	int localExtractedLength = 0;
	char *localSeqPtr = NULL;
	DataTypeDescriptor dtd;
	int sequenceLength = 0;
	ServiceRecord *svcRec = NULL;
	ServiceAttributeIdentifier lookAheadAttrId = 0xffff;

	localSeqPtr = extractSequenceType(pdata, &dtd, &sequenceLength, 
			bytesScanned);
	lookAheadAttrId = ntohs(__get_u16(localSeqPtr +	sizeof(DataTypeDescriptor)));
	BTDEBUG("Look ahead attr id : %d", lookAheadAttrId);

	/*
	 ** Check if the PDU already has a service record handle,
	 ** int which case this may be a update request
	 */
	if (handleExpected != 0xffffffff)
		svcRec = findServiceRecord(handleExpected);
	
	if (svcRec == NULL) {
		svcRec = createServiceRecord();
		svcRec->attributeList = NULL;
	}

	localExtractedLength = extractAttributes(svcRec, localSeqPtr, sequenceLength);
	if (localExtractedLength >= 0)
		*bytesScanned += localExtractedLength;

	return svcRec;
}

/* 
 * Extract PDU for Client
 */
ServiceRecord *clientExtractPDU(char *pdata, ServiceRecordHandle handleExpected, int *bytesScanned)
{
	int localExtractedLength = 0;
	char *localSeqPtr = NULL;
	DataTypeDescriptor dtd;
	int sequenceLength = 0;
	ServiceRecord *svcRec = NULL;
	ServiceAttributeIdentifier lookAheadAttrId = 0xffff;
	ServiceRecordHandle svcRecHandle = 0xffffffff;

	localSeqPtr = extractSequenceType(pdata, &dtd, &sequenceLength, 
			bytesScanned);
	lookAheadAttrId = ntohs(__get_u16(localSeqPtr + sizeof(DataTypeDescriptor)));
	BTDEBUG("Look ahead attr id : %d", lookAheadAttrId);

	/*
	 ** Check if the PDU already has a service record handle,
	 ** int which case this may be a update request
	 */
	if (lookAheadAttrId == AttrServiceRecordHandle) {
		svcRecHandle = ntohl(__get_u32(localSeqPtr + 
					sizeof(DataTypeDescriptor) +
					sizeof(ServiceAttributeIdentifier) +
					sizeof(DataTypeDescriptor)));
		BTDEBUG("SvcRecHandle : 0x%x", svcRecHandle);
	}

	svcRec = createServiceRecord();
	if (svcRec == NULL)
		return NULL;
	svcRec->attributeList = NULL;
	if (lookAheadAttrId == AttrServiceRecordHandle) {
		svcRec->serviceRecordHandle = svcRecHandle;
	} else {
		if (handleExpected != 0xffffffff) {
			svcRec->serviceRecordHandle = handleExpected;
		} else {
			BTINFO("ServiceRecordHandle not defined");
		}
	}

	localExtractedLength = extractAttributes(svcRec, localSeqPtr, sequenceLength);
	if (localExtractedLength >= 0)
		*bytesScanned += localExtractedLength;

	return svcRec;
}


void printInteger(SDPData *pData)
{
	if (pData != NULL)
		BTDEBUG("Integer : 0x%x", pData->sdpDataValue.uint32);
}

void printString(SDPData *pData)
{
	if (pData != NULL)
		BTDEBUG("String : %s", pData->sdpDataValue.stringPtr);
}

void printDataSeq(SDPData *pData)
{
	SDPData *pSDPData = NULL;

	pSDPData = pData;
	if (pSDPData != NULL) {
		do {
			printSDPData(pSDPData);
			pSDPData = pSDPData->next;
		} while (pSDPData != NULL);
	}
	else {
		BTDEBUG("Broken dataSeq link");
	}
}

void printDataSeqAlternates(SDPData *pData)
{
	int i = 0;
	SDPData *pSDPData = NULL;

	if (pData != NULL) {
		BTDEBUG("Alt count : %d", pData->alternatesCount);
		while (i < pData->alternatesCount) {
			pSDPData = pData->alternates[i];
			if (pSDPData != NULL) {
				printSDPData(pSDPData); 
			}
			i++;
		}
	}
}

void printSDPDataAsServiceRecord(ServiceRecord *svcRec)
{
	ServiceAttributeIdentifier attrId;
	SDPData *pSDPData;

	/*
	// print the service record handle 
	attrId = AttrServiceRecordHandle;
	pSDPData = getSDPData(svcRec, attrId);   
	if (pSDPData != NULL) {
	printf("Service Record Handle: 0x%x\n", pSDPData->sdpDataValue.uint32);
	}
	*/

	// print the service name
	attrId = AttrServiceName_Primary;
	pSDPData = getSDPData(svcRec, attrId);   
	if (pSDPData != NULL) {
		printf("Service Name: %s\n", pSDPData->sdpDataValue.stringPtr);
	}
	attrId = AttrServiceDescription_Primary;
	pSDPData = getSDPData(svcRec, attrId);   
	if (pSDPData != NULL) {
		printf("Service Description: %s\n", pSDPData->sdpDataValue.stringPtr);
	}
	attrId = AttrServiceProviderName_Primary;
	pSDPData = getSDPData(svcRec, attrId);   
	if (pSDPData != NULL) {
		printf("Service Provider: %s\n", pSDPData->sdpDataValue.stringPtr);
	}
}
void printSDPData(SDPData *pSDPData)
{
	DataTypeDescriptor dtd;

	dtd = pSDPData->dtd;
	switch (dtd) {
		case Boolean :
		case UnsignedInteger_8 :
		case UnsignedInteger_16 :
		case UnsignedInteger_32 :
		case UnsignedInteger_64 :
		case UnsignedInteger_128 :
		case Integer_8 :
		case Integer_16 :
		case Integer_32 :
		case Integer_64 :
		case Integer_128 :
			printInteger(pSDPData);
			break;

		case UUID_16 :
		case UUID_32 :
		case UUID_128 :
			BTDEBUG("UUID");
			printUUID(&pSDPData->sdpDataValue.uuid);
			break;

		case TextStringSize8 :
		case TextStringSize16 :
		case TextStringSize32 :
			BTDEBUG("Text : %s", pSDPData->sdpDataValue.stringPtr);
			break;
		case URLStringSize8 :
		case URLStringSize16 :
		case URLStringSize32 :
			BTDEBUG("URL : %s", pSDPData->sdpDataValue.stringPtr);
			break;

		case SequenceSize8 :
		case SequenceSize16 :
		case SequenceSize32 :
			printDataSeq(pSDPData->sdpDataValue.dataSeq);
			break;

		case SequenceAlternatesSize8 :
		case SequenceAlternatesSize16 :
		case SequenceAlternatesSize32 :
			BTDEBUG("Data Sequence Alternates");
			printDataSeq(pSDPData->sdpDataValue.dataSeq);
			printDataSeqAlternates(pSDPData);
			break;
	}
}

void attrPrintFunc(gpointer value, gpointer userData)
{
	SDPData *pSDPData = NULL;
	ServiceAttributeIdentifier attrId;

	BTDEBUG("[%s-%d] : ", __FUNCTION__, __LINE__);
	pSDPData = (SDPData *)value;
	attrId = pSDPData->attrId;
	BTDEBUG("=====================================");
	BTDEBUG("ATTRIBUTE IDENTIFIER : 0x%x", attrId);
	BTDEBUG("ATTRIBUTE VALUE PTR : 0x%x", (uint32_t)value);
	if (pSDPData != NULL) {
		printSDPData(pSDPData);
	}
	else
		BTDEBUG("NULL value");
	BTDEBUG("=====================================");
}

void printServiceAttributes(GSList *svcAttrList)
{
	g_slist_foreach(svcAttrList, attrPrintFunc, NULL);
}


SDPData *getSDPData(ServiceRecord *svcRec, ServiceAttributeIdentifier attrId)
{
	SDPData *pSDPData = NULL;
	SDPData sdpTemplate;

	if ((svcRec != NULL) && (svcRec->attributeList != NULL)) {
		GSList *pStartElem = NULL;
		sdpTemplate.attrId = attrId;
		pStartElem = g_slist_find_custom(svcRec->attributeList,
				&sdpTemplate, attrIdCompareFunc); 
		if (pStartElem != NULL)
			pSDPData = (SDPData *)pStartElem->data;
	}

	return pSDPData;
}


