/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author:Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   */

/*
   $Id: lock.c,v 1.2 2002/08/30 11:18:34 kds Exp $

   Contains the implementation of readers/writer lock to
   be used while accessing service repository

	Fixes:     Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>
*/

#include <affix/config.h>

#include <stdio.h>
#include <pthread.h>
#include "utils.h"

pthread_rwlock_t sdpRWLock;
short readCount = 0;

/*
 ** Initialize the readers/writers lock machinery.
 ** We use the pthreads library provided mechanism
 */
void initLock(void)
{
	pthread_rwlock_init(&sdpRWLock, NULL);
}


/*
 ** Acquire the readers lock
 */
void acquireReadersLock(void)
{
	BTDEBUG("RDL Try thr : %ld readCount : %d", pthread_self(), readCount);
	pthread_rwlock_rdlock(&sdpRWLock);
	readCount++;
	BTDEBUG("RDL Acq thr : %ld readCount : %d", pthread_self(), readCount);
}


/*
 ** Release the readers lock
 */
void releaseReadersLock(void)
{
	readCount--;
	pthread_rwlock_unlock(&sdpRWLock);
	BTDEBUG("RDL Rel thr : %ld readCount : %d", pthread_self(), readCount);
}


/*
 ** Acquire the writers lock
 */
void acquireWritersLock(void)
{
	BTDEBUG("WRL Try thr : %ld readCount : %d", pthread_self(), readCount);
	pthread_rwlock_wrlock(&sdpRWLock);
	BTDEBUG("WRL Acq thr : %ld readCount : %d", pthread_self(), readCount);
}


/*
 ** Release the writers lock
 */
void releaseWritersLock(void)
{
	pthread_rwlock_unlock(&sdpRWLock);
	BTDEBUG("WRL Rel thr : %ld readCount : %d", pthread_self(), readCount);
}
