/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author:Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   */

/*
   $Id: queue.c,v 1.2 2002/08/30 11:18:34 kds Exp $

   Contains the queue implementation. Client requests (both
   service discovery and registration) are queued and picked
   off and processed by a group of worker threads

	Fixes:     Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>
*/

#include <affix/config.h>

#include <pthread.h>
#include "queue.h"


static GSList			*requestQueue_ = NULL;
static pthread_mutex_t		mutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t		cond = PTHREAD_COND_INITIALIZER;


/*
 ** Setup the queue and initialize data structures
 */
int initQueue()
{
	return 0;
}


/*
 ** Add an entry to the queue. Grab the lock, append
 ** and signal on conditional to wake-up waiting threads
 */
int addEntryToRequestQueue(SDPRequest *sdpRequest)
{
	pthread_mutex_lock(&mutex);
	/*
	 ** Do stuff ..
	 */
	requestQueue_ = g_slist_append(requestQueue_, sdpRequest);
	pthread_cond_signal(&cond);
	pthread_mutex_unlock(&mutex);

	if (requestQueue_ != NULL)
		return 0;
	else
		return -1;
}

/*
 ** Take the first entry from the queue.
 ** Usually called by the "free" worker thread
 */
SDPRequest *removeFirstEntry()
{
	gpointer ptr;

	pthread_mutex_lock(&mutex);
	/*
	 ** Do stuff ..
	 */
	while ((ptr = g_slist_nth_data(requestQueue_, 0)) == NULL) {
		pthread_cond_wait(&cond, &mutex);
	}
	requestQueue_ = g_slist_remove(requestQueue_, ptr);
	pthread_mutex_unlock(&mutex);

	return (SDPRequest *)ptr;
}

/*
 ** Find the number entries in the queue
 */
int queueLength()
{
	int length;

	pthread_mutex_lock(&mutex);
	/*
	 ** Do stuff ..
	 */
	length = g_slist_length(requestQueue_);
	pthread_mutex_unlock(&mutex);

	return length;
}

/*
 ** Cleanup queue contents
 ** Called when server is about to shutdown
 */
void flushQueue()
{
}

