/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author:Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   */

/*
   $Id: record.c,v 1.1.1.1 2002/07/11 11:21:57 kds Exp $

   Contains all the code for maintaining a service record.
   The service record is the fundamental service attribute(s)
   aggregator. There is one service record per registered service

Fixes:     Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>
*/

#include <affix/config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <affix/sdp.h>
#include <affix/sdpclt.h>
#include "utils.h"
#include "servicedb.h"
#include "match.h"

ServiceRecord *sdpServiceRecord;

/*
 ** This version is for use by SDP server when creating new
 ** service records
 */
ServiceRecord *createServiceRecord(void)
{
	ServiceRecord *svcRec;

	/*
	 ** Create an empty ServiceRecord, but with 
	 ** ServiceRecordHandle set to 0x0000 which
	 ** is the handle for the SDP server itself.
	 **
	 ** The server main program does the rest of
	 ** setting other attributes.
	 */
	svcRec = (ServiceRecord *)malloc(sizeof(ServiceRecord));
	memset((void *)svcRec, 0, sizeof(ServiceRecord));

	svcRec->serviceRecordHandle = 0xffffffff;

	return svcRec;
}



/*
 ** Print the contents of a service record
 */
void printServiceRecord(BD_ADDR bdAddr, ServiceRecordHandle svcRecHandle)
{
}

void freeServiceRecordList(GSList *svcRecList)
{
	int	len, i;
	ServiceRecord	*svcRec;
	
	len = g_slist_length(svcRecList);
	for (i = 0; i < len; i++) {
		svcRec = (ServiceRecord *)g_slist_nth_data(svcRecList, i);
		if (svcRec != NULL) {
			BTDEBUG("Freeing : 0x%x\n", svcRec->serviceRecordHandle);
			freeServiceRecord(svcRec);
		}
	}
}

/*
 ** Free the contents of a service record
 */
void freeServiceRecord(ServiceRecord *svcRec)
{
	if (svcRec != NULL) {
		free(svcRec->pduForm.data);
		freeSList(svcRec->targetPattern);
		free(svcRec);
	}
}

void removeUUIDFromTargetPattern(ServiceRecord *svcRec, GSList *pSeq)
{
}

void addUUIDToTargetPattern(ServiceRecord *svcRec, UUID *pUUID)
{
	UUID *pUUID128Bit;

	if ((svcRec != NULL) && (pUUID != NULL)) {
		pUUID128Bit = convertTo128Bits(pUUID);
		BTDEBUG("SvcRec : 0x%lx", (unsigned long)svcRec);
		BTDEBUG("Elements in target pattern : %d", g_slist_length(svcRec->targetPattern));
		BTDEBUG("Trying to add : 0x%lx", (unsigned long)pUUID128Bit);
		if (g_slist_find_custom(svcRec->targetPattern, pUUID128Bit, compareUUID) == NULL)
			svcRec->targetPattern = g_slist_insert_sorted(
					svcRec->targetPattern, pUUID128Bit, compareUUID);
		else
			free(pUUID128Bit);
		BTDEBUG("Elements in target pattern : %d", g_slist_length(svcRec->targetPattern));
	}
}

void addUUIDSeqToTargetPattern(ServiceRecord *svcRec, GSList *pSeq)
{
	int index = 0;

	while (1) {
		UUID *pUUID = (UUID *)g_slist_nth_data(pSeq, index);
		index++;
		if (pUUID != NULL) {
			addUUIDToTargetPattern(svcRec, pUUID);
		}
		else
			break;
	}              
}

/*
 ** A sequence is captured by a GSList. Every element of the
 ** GSList points to things which need to be free()'d. This
 ** method accomplishes freeing the contents of a GSList
 */
void freeSList(GSList *pSeq)
{
	int index = 0;

	while (1) {
		gpointer data;

		data = g_slist_nth_data(pSeq, index);
		if (data == NULL)
			break;
		else {
			free(data); index++;
		}
	}
	if (pSeq != NULL)
		g_slist_free(pSeq);
}

