/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author:Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
   $Id: sdpsrv_utils.c,v 1.1.1.1 2002/07/11 11:21:57 kds Exp $

   Contains utlities for use by SDP service registration clients
   to add attributes to a service record. 
 
   Fixes:     Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>
*/
	 
#include <affix/config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <affix/sdp.h>
#include <affix/sdpsrv.h>
#include "servicedb.h"

#include "attr.h"
#include "utils.h"

ServiceAttributeIdentifier svcClassAttrId = AttrServiceClassIDList;
ServiceAttributeIdentifier browseGroupAttrId = AttrBrowseGroupList;
ServiceAttributeIdentifier profileDescAttrId = AttrBTProfileDescriptorList;
ServiceAttributeIdentifier svcTTLAttrId = AttrServiceInfoTimeToLive;
ServiceAttributeIdentifier svcRecStateAttrId = AttrServiceRecordState;
ServiceAttributeIdentifier svcIDAttrId = AttrServiceID;
ServiceAttributeIdentifier svcGroupAttrId = AttrGroupID;
ServiceAttributeIdentifier svcAvailAttrId = AttrServiceAvailability;
ServiceAttributeIdentifier svcProtDescAttrId = AttrProtocolDescriptorList;

ServiceAttributeIdentifier svcNameAttrId = AttrServiceName_Primary;
ServiceAttributeIdentifier svcProvNameAttrId = AttrServiceDescription_Primary;
ServiceAttributeIdentifier svcDescAttrId = AttrServiceProviderName_Primary;
ServiceAttributeIdentifier clientExecURLId = AttrClientExecutableURL;
ServiceAttributeIdentifier docURLId = AttrDocumentationURL;
ServiceAttributeIdentifier iconURLId = AttrIconURL;

/*
** NOTE that none of the setXXX() functions below will
** actually update the SDP server, unless the
** {register, update}ServiceRecord() function is invoked.
**
**
*/


void setInternalState(ServiceRecord *svcRec)
{
  ServiceAttributeIdentifier attrSvcClass = AttrServiceClassIDList;
  ServiceAttributeIdentifier attrPDList = AttrProtocolDescriptorList;
  ServiceAttributeIdentifier attrGroup = AttrGroupID;

  BTDEBUG("Internal state of svcRec : %d", svcRec->internalState);
  if (svcRec->internalState == created) {
    if ((getSDPData(svcRec, attrSvcClass) != NULL) &&
        (getSDPData(svcRec, attrPDList) != NULL)) {
      svcRec->internalState = createdRegisterReady;
      BTDEBUG("Internal state of svcRec reg ready : %d", svcRec->internalState);
    }
    else
    /*
    ** This could be a browse group descriptor, which
    ** need not contain mandatory attributes
    */
    if (getSDPData(svcRec, attrGroup) != NULL) {
      svcRec->internalState = createdRegisterReady;
      BTDEBUG("Internal state of svcRec reg ready : %d", svcRec->internalState);
    }
  }
  else 
  if (svcRec->internalState == registered) {
    svcRec->internalState = modifiedUpdateReady;
  }
}

int addBasicAttribute(ServiceRecord *svcRec, DataTypeDescriptor dtd, void *pValue, ServiceAttributeIdentifier attributeId)
{
  int status = 0;

  if ((svcRec != NULL) && (pValue != NULL)) {
    SDPData *pSDPData = createSDPData(dtd, (char *)pValue);
    if (pSDPData != NULL) {
      pSDPData->attrId = attributeId;
      attrPduForm(pSDPData, NULL);
      /*
      ** Replace old value
      */
      addToAttributeList(svcRec, attributeId, pSDPData, 1);
      setInternalState(svcRec);
    }
    else
      status = -1;
  }
  else
    status = -1;

  return status;
}


/*
** Set the information attributes of the service
** pointed to by svcRecHandle. The set of attributes
** include service name, description and provider name
**
** Returns 0 if successful and -1 if there is
** a failure.
*/
int setInformationalAttributes(
  ServiceRecord *svcRec,
  char *serviceName,
  char *providerName,
  char *serviceDescription
)
{
  int status = 0;
  DataTypeDescriptor dtd = TextStringSize8;

  if ((svcRec != NULL) && (svcRec->internalState != doesNotExist)) {
    addBasicAttribute(svcRec, dtd, serviceName, svcNameAttrId);
    addBasicAttribute(svcRec, dtd, providerName, svcProvNameAttrId);
    addBasicAttribute(svcRec, dtd, serviceDescription, svcDescAttrId);
    setInternalState(svcRec);
  }
  else
    status = -1;

  return status;
}


#define addUUIDSequenceAttribute(svcRec, attributeId, pSeq)	\
{								\
  int status = 0, i;						\
  int length;							\
  SDPData *pSequenceHolder = NULL;				\
  GPtrArray *dtdArray;						\
  GPtrArray *valueArray;					\
  DataTypeDescriptor uuid16 = UUID_16;				\
  DataTypeDescriptor uuid32 = UUID_32;				\
  DataTypeDescriptor uuid128 = UUID_128;			\
								\
  length = g_slist_length(pSeq);				\
  if ((pSeq != NULL) && (length > 0)) {				\
								\
    dtdArray = g_ptr_array_new();				\
    valueArray = g_ptr_array_new();				\
    for (i = 0; i < length; i++) { 				\
      UUID *pUUID = (UUID *)g_slist_nth_data(pSeq, i);		\
      if (pUUID != NULL) {					\
        switch (pUUID->uuidType) {				\
          case bits_16 :					\
            g_ptr_array_add(dtdArray, &uuid16);			\
            g_ptr_array_add(valueArray,         		\
              (char *)&pUUID->value.uuid16Bit);			\
            break;						\
          case bits_32 :					\
            g_ptr_array_add(dtdArray, &uuid32);			\
            g_ptr_array_add(valueArray, 			\
	      (char *)&pUUID->value.uuid32Bit);			\
            break;						\
          case bits_128 :					\
            g_ptr_array_add(dtdArray, &uuid128);			\
            g_ptr_array_add(valueArray, 			\
	      (char *)&pUUID->value.uuid128Bit);		\
            break;						\
          default :						\
            status = -1;					\
            break;						\
        }							\
      }								\
      else {							\
        status = -1;						\
        break;							\
      }								\
    }								\
								\
    if (status == 0) 						\
      pSequenceHolder = createSDPDataSequence(dtdArray,		\
	valueArray);						\
    if (pSequenceHolder != NULL) {				\
      pSequenceHolder->attrId = attributeId;			\
      addToAttributeList(svcRec, attributeId, 			\
        pSequenceHolder, 1);					\
      setInternalState(svcRec);					\
      addUUIDSeqToTargetPattern(svcRec, pSeq);			\
    }								\
    g_ptr_array_free(dtdArray, FALSE); 				\
    g_ptr_array_free(valueArray, FALSE);			\
  }								\
								\
  return status;						\
}					


/*
** This function sets the serviceClassID of the
** service pointed to by ServiceRecord,
** to the list specified by svcClassIDList.
** Note that the identifiers need to be in
** sorted order from the most specific to the
** most generic service class that this service
** conforms to.
**
** Using a value of NULL for svcClassIDList has
** effect of removing this attribute (if previously set)
** 
** This function replaces the existing ClassID
** list with a new one specified by svcClassIDList
** if one is present.
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setServiceClassID(
  ServiceRecord *svcRec,
  GSList *svcClassIDList
)
{
  addUUIDSequenceAttribute(svcRec, svcClassAttrId, svcClassIDList);
}


/*
** This function sets the BrowseGroupList of the
** service pointed to by ServiceRecord,
** to the list specified by browseGroupSeq.
**
** Using a value of NULL for browseGroupSeq has
** effect of removing this attribute (if previously set)
** 
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setBrowseGroupList(
  ServiceRecord *svcRec,
  GSList *browseGroupSeq
)
{
  addUUIDSequenceAttribute(svcRec, browseGroupAttrId, browseGroupSeq);
}


/*
** Each entry in the GSList passed to this
** function as argument, contains a ProtocolDescriptor
** object
*/
SDPData *accessProtocolsToDataSeq(
  ServiceRecord *svcRec,
  GSList *pDescSeq
)
{
  int status = 0;
  SDPData *pSeq = NULL;
  SDPData *pAPSeq = NULL;
  GPtrArray *dtdArray = NULL;
  GPtrArray *seqDTDArray = NULL;
  GPtrArray *valueArray = NULL;
  GPtrArray *seqArray = NULL;
  DataTypeDescriptor uuid16 = UUID_16;
  DataTypeDescriptor uuid32 = UUID_32;
  DataTypeDescriptor uuid128 = UUID_128;
  DataTypeDescriptor uint8 = UnsignedInteger_8;
  DataTypeDescriptor uint16 = UnsignedInteger_16;

  if (pDescSeq != NULL) {
    int i = 0;
    ProtocolDescriptor *pDesc;

    int seqLength = g_slist_length(pDescSeq);
    seqDTDArray = g_ptr_array_new();
    seqArray = g_ptr_array_new();
    while (i < seqLength) {
      dtdArray = g_ptr_array_new();
      valueArray = g_ptr_array_new();
      pDesc = (ProtocolDescriptor *)g_slist_nth_data(pDescSeq, i);
      if (pDesc != NULL) {
        /*
        ** Mandatory attribute in a ProtocolDescriptor is
        ** the protocol id (UUID), followed optionally by
        ** a port number, and a version number
        */
        switch (pDesc->protocolId.uuidType) {
          case bits_16 :
            g_ptr_array_add(dtdArray, &uuid16);
            g_ptr_array_add(valueArray, &pDesc->protocolId.value.uuid16Bit);
            break;
          case bits_32 :
            g_ptr_array_add(dtdArray, &uuid32);
            g_ptr_array_add(valueArray, &pDesc->protocolId.value.uuid32Bit);
            break;
          case bits_128 :
            g_ptr_array_add(dtdArray, &uuid128);
            g_ptr_array_add(valueArray, &pDesc->protocolId.value.uuid128Bit);
            break;
          default :
            status = -1;
            break;
        }
        if (pDesc->portNumber != 0xffff) {
          if (pDesc->portNumberSize == 1)
            g_ptr_array_add(dtdArray, &uint8);
          else
            g_ptr_array_add(dtdArray, &uint16);
          g_ptr_array_add(valueArray, &pDesc->portNumber);
        }
        if (pDesc->versionNumber != 0xffff) {
          g_ptr_array_add(dtdArray, &uint16);
          g_ptr_array_add(valueArray, &pDesc->versionNumber);
        }
        if (status == 0) {
          pSeq = createSDPDataSequence(dtdArray, valueArray);
          if (pSeq != NULL) {
            g_ptr_array_add(seqDTDArray, &pSeq->dtd);
            g_ptr_array_add(seqArray, pSeq);
            addUUIDToTargetPattern(svcRec, &pDesc->protocolId);
          }
        }
      }
      else {
        status = -1;
        break;
      }
      i++;
      g_ptr_array_free(dtdArray, TRUE);
      g_ptr_array_free(valueArray, TRUE);
    }
    if (status == 0) 
      pAPSeq = createSDPDataSequence(seqDTDArray, seqArray);

    g_ptr_array_free(seqDTDArray, TRUE);
    g_ptr_array_free(seqArray, TRUE);
  }

  return pAPSeq;
}


/*
** This function sets the access protocols
** of the service specified by ServiceRecord.
** to the value specified in "accessProtocols"
**
** Note that if there are alternate mechanisms by 
** which the service is accessed, then they should 
** be specified as sequences 
**
** Using a value of NULL for acessProtocols has
** effect of removing this attribute (if previously set)
** 
** This function replaces the exisiting AccessProtocols
** structure (if any) with the new one specified.
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setAccessProtocols(
  ServiceRecord *svcRec,
  AccessProtocols *ap
)
{
  int status = 0, i;
  SDPData *pOriginal = NULL;
  SDPData *pSeq = NULL;
  int size = 0;
  GSList *currEntry;
  
  if (ap != NULL) {
    size = g_slist_length(ap->pSequence);
  }
  if ((svcRec != NULL) && (size > 0)) {
    for (i = 0; i < size; i++) {
      currEntry = (GSList *)g_slist_nth_data(ap->pSequence, i);
      pSeq = accessProtocolsToDataSeq(svcRec,currEntry);
      if (pSeq != NULL) {
        if (i == 0) {
          pOriginal = pSeq;
        }
        else {
          addSDPAlternateData(pOriginal, pSeq);
        }
      }
      else {
        status = -1;
        break;
      }
    }
    if (status == 0) {
      createSDPAttribute(svcRec, svcProtDescAttrId, pOriginal);
    }
  }

  if ((status == -1) && (pOriginal != NULL)) {
    freeSDPData(pOriginal);
  }

  return status;
}


/*
** This function sets the "LanguageBase"
** attributes of the service record pointed
** to by svcRecHandle to the value specified 
** in "langAttrList".
**
** "langAttrList" is a linked list of "LanguageAttributes"
** objects, one for each language in which user visible
** attributes are present in the service record.
**
** Using a value of NULL for langAttrList has
** effect of removing this attribute (if previously set)
** 
** This function replaces the exisiting LanguageAttributes
** structure (if any) with the new one specified.
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setLanguageBasedAttributes(
  ServiceRecord *svcRec,
  GSList *lSeq
)
{
  return 0;
}
 

/*
** This function sets the "ServiceInfoTimeToLive"
** attribute of the service. This is the number
** of seconds that this service record is guaranteed
** not to change after the record was obtained by the
** client from the SDP server.
**
** It is not clear if this attribute
** is set by the SDP server or the owner or the
** service record. Just in case ..
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setServiceInfoTimeToLive(
  ServiceRecord *svcRec,
  ServiceInfoTimeToLive svcTTL
)
{
  int status = 0;

  addBasicAttribute(svcRec, UnsignedInteger_32, &svcTTL, svcTTLAttrId);

  return status;
}


/*
** This function sets the "ServiceRecordState"
** attribute of the service. This attribute is
** guaranteed to change if there is any kind of
** modification to the service record. 
** 
** Again it is not clear if this attribute is set
** by the SDP server or the service provider. But
** this one seems more likely an attribute that
** is set by the service provider.
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setServiceRecordState(
  ServiceRecord *svcRec,
  ServiceRecordState svcRecState
)
{
  int status = 0;

  addBasicAttribute(svcRec, UnsignedInteger_32, &svcRecState, 
                    svcRecStateAttrId);
  return status;
}


/*
** This function sets the "ServiceID"
** attribute of the service. 
** 
** This is the UUID of the service. 
** 
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setServiceID(
  ServiceRecord *svcRec,
  UUID svcUUID
)
{
  int status = 0;

  switch (svcUUID.uuidType) {
    case bits_16 :
      addBasicAttribute(svcRec, UUID_16, &svcUUID.value.uuid16Bit, 
                        svcIDAttrId);
      break;
    case bits_32 :
      addBasicAttribute(svcRec, UUID_32, &svcUUID.value.uuid32Bit, 
                        svcIDAttrId);
      break;
    case bits_128 :
      addBasicAttribute(svcRec, UUID_128, &svcUUID.value.uuid128Bit, 
                        svcIDAttrId);
      break;
  }
  addUUIDToTargetPattern(svcRec, &svcUUID);

  return status;
}


/*
** This function sets the "GroupID"
** attribute of the service record
** defining a group. 
** 
** This is the UUID of the group. 
** 
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setGroupID(
  ServiceRecord *svcRec,
  UUID groupUUID
)
{
  int status = 0;

  switch (groupUUID.uuidType) {
    case bits_16 :
      addBasicAttribute(svcRec, UUID_16, &groupUUID.value.uuid16Bit, 
                        svcGroupAttrId);
      break;
    case bits_32 :
      addBasicAttribute(svcRec, UUID_32, &groupUUID.value.uuid32Bit, 
                        svcGroupAttrId);
      break;
    case bits_128 :
      addBasicAttribute(svcRec, UUID_128, &groupUUID.value.uuid128Bit, 
                        svcGroupAttrId);
      break;
  }
  addUUIDToTargetPattern(svcRec, &groupUUID);

  return status;
}


/*
** This function sets the "ServiceAvailability"
** attribute of the service pointed to svcRecHandle
** to the value specified in svcAvail.
**
** Note that this represents the relative availability
** of the service 0x00 means compeletely unavailable to
** 0xFF meaning maximum availability.
** 
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setServiceAvailability(
  ServiceRecord *svcRec,
  ServiceAvailability svcAvail
)
{
  int status = 0;

  addBasicAttribute(svcRec, UnsignedInteger_8, &svcAvail, svcAvailAttrId);

  return status;
}


/*
** This function sets the "BTProfileDescriptorList"
** attribute of the service record pointed
** to by svcRecHandle to the value specified 
** in "profileDesc".
**
** Each element in the GSList is an object of type
** ProfileDescriptor which is a definition of the
** Bluetooth profile that this service conforms to.
**
** Using a value of NULL for profileDesc has
** effect of removing this attribute (if previously set)
** 
** This function replaces the exisiting BTProfileDescritorList
** structure (if any) with the new one specified.
**
** Returns 0 if successful or
** -1 if there is a failure.
**
*/
int setBTProfileDescriptorList(
  ServiceRecord *svcRec,
  GSList *profileDescSeq
)
{
  int status = 0;
  SDPData *pSeq = NULL;
  SDPData *pAPSeq = NULL;
  GPtrArray *dtdArray = NULL;
  GPtrArray *seqDTDArray = NULL;
  GPtrArray *valueArray = NULL;
  GPtrArray *seqArray = NULL;
  DataTypeDescriptor uuid16 = UUID_16;
  DataTypeDescriptor uuid32 = UUID_32;
  DataTypeDescriptor uuid128 = UUID_128;
  DataTypeDescriptor uint16 = UnsignedInteger_16;

  if (profileDescSeq != NULL) {
    int i = 0;
    ProfileDescriptor *pDesc;

    int seqLength = g_slist_length(profileDescSeq);
    seqDTDArray = g_ptr_array_new();
    seqArray = g_ptr_array_new();
    while (i < seqLength) {
      dtdArray = g_ptr_array_new();
      valueArray = g_ptr_array_new();
      pDesc = (ProfileDescriptor *)g_slist_nth_data(profileDescSeq, i);
      if (pDesc != NULL) {
        /*
        ** Mandatory attribute in a ProtocolDescriptor is
        ** the protocol id (UUID), followed optionally by
        ** a port number, and a version number
        */
        switch (pDesc->profileUUID.uuidType) {
          case bits_16 :
            g_ptr_array_add(dtdArray, &uuid16);
            g_ptr_array_add(valueArray, &pDesc->profileUUID.value.uuid16Bit);
            break;
          case bits_32 :
            g_ptr_array_add(dtdArray, &uuid32);
            g_ptr_array_add(valueArray, &pDesc->profileUUID.value.uuid32Bit);
            break;
          case bits_128 :
            g_ptr_array_add(dtdArray, &uuid128);
            g_ptr_array_add(valueArray, &pDesc->profileUUID.value.uuid128Bit);
            break;
          default :
            status = -1;
            break;
        }
        g_ptr_array_add(dtdArray, &uint16);
        g_ptr_array_add(valueArray, &pDesc->vnum);
        pSeq = createSDPDataSequence(dtdArray, valueArray);
        if (pSeq != NULL) {
            g_ptr_array_add(seqDTDArray, &pSeq->dtd);
            g_ptr_array_add(seqArray, pSeq);
            addUUIDToTargetPattern(svcRec, &pDesc->profileUUID);
        }
      }
      else {
        status = -1;
        break;
      }
      i++;
      g_ptr_array_free(dtdArray, TRUE);
      g_ptr_array_free(valueArray, TRUE);
    }
    if (status == 0) 
    {
      pAPSeq = createSDPDataSequence(seqDTDArray, seqArray);
      status = createSDPAttribute(svcRec, AttrBTProfileDescriptorList, pAPSeq);
    }

    g_ptr_array_free(seqDTDArray, TRUE);
    g_ptr_array_free(seqArray, TRUE);
  }
  return status;
  
}
 

/*
** Sets various URL attributes of the service
** pointed to by svcRecHandle. The URL include
**
** ClientExecutableURL : a URL to the client's
**   platform specific (WinCE, PalmOS) executable
**   code that can be used to access this service.
**
** DocumentationURL : a URL pointing to service documentation
**
** IconURL : a URL to an icon that can be used to represent
**   this service.
**
** Note that you need to pass NULL for any URLs
** that you don't want to set or remove
**
** Returns 0 if successful and -1 if there is
** a failure.
**
*/
int setURLAttributes(
  ServiceRecord *svcRec,
  char *clientExecURL,
  char *docURL,
  char *iconURL
)
{
  int status = 0;
  DataTypeDescriptor dtd;

  dtd = URLStringSize8;

  if ((svcRec != NULL) && (svcRec->internalState != doesNotExist)) {
    addBasicAttribute(svcRec, dtd, clientExecURL, clientExecURLId);
    addBasicAttribute(svcRec, dtd, docURL, docURLId);
    addBasicAttribute(svcRec, dtd, iconURL, iconURLId);
    setInternalState(svcRec);
  }
  else 
    status = -1;

  return status;
}


