/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author:Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
   $Id: servicedb.h,v 1.2 2002/08/01 17:52:08 kds Exp $

   Contains methods to add/find/modify/delete entries in the service
   repository

   Fixes:     Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>
*/

#ifndef SDP_SERVICE_DB_H
#define SDP_SERVICE_DB_H

#include <affix/sdp.h>

/*
** Cleans up the service table
**
*/
void initServiceDB(void);


/*
** Purge the DB of all service record(s).
** Happens when SDP server is about to
** shutdown
*/
void resetServiceDB(void);


/*
** Add a new service record to the repository
*/
int addServiceRecord(ServiceRecord *svcRecord);


/*
** Find a service record given it's handle
*/
ServiceRecord *findServiceRecord(ServiceRecordHandle svcHandle);


/*
** Remove a service record given it's handle
*/
int removeServiceRecord(ServiceRecordHandle svcHandle);


/*
** Get a list containing a handle to all registered services
*/
GSList *getServiceRecordList(BD_ADDR bdAddr);


int emptyServiceDB(void);
int removeServiceRecordByFd(int fd);

#endif //SDP_SERVICE_DB_H
