/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author:Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   */

/*
   $Id: uuid.c,v 1.2 2002/07/19 15:18:34 kds Exp $

	Fixes:	Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>
	Manel Guerrero Zapata <manel.guerrero-zapata@nokia.com>

	Dmitry Kasatkin		- added UUID32 to string conversion
*/

#include <affix/config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>

#include <affix/sdp.h>
#include <affix/sdpclt.h>


UUID128Bit *pBluetoothBaseUUID128Bit = NULL;

/* Message structure. */
struct tupla
{
	int index;
	char *str;
};

struct tupla ProtocolMnemonic[] = {
	{SDP_UUID,		"SDP"},
	{RFCOMM_UUID,		"RFCOMM"},
	{TCS_BIN_UUID,		"TCS-BIN"},
	{L2CAP_UUID,		"L2CAP"},
	{IP_UUID,		"IP"},
	{UDP_UUID,		"UDP"},
	{TCP_UUID,		"TCP"},
	{TCS_AT_UUID,		"TCS-AT"},
	{OBEX_UUID,		"OBEX"},	
	{FTP_UUID,		"FTP"},
	{HTTP_UUID,		"HTTP"},
	{WSP_UUID,		"WSP"},
};

struct tupla ServiceClassMnemonic[] = {
	{SDPServerServiceClassID,		"SDP Server"},
	{BrowseGroupDescServiceClassID,		"Browse Group Descriptor"},
	{PublicBrowseGroup,			"Public Browse Group"},
	{SerialPortServiceClassID,		"Serial Port"},
	{LANAccessUsingPPPServiceClassID,	"LAN Access Using PPP"},
	{DialupNetworkingServiceClassID,	"Dialup Networking"},
	{IrMCSyncServiceClassID,		"IrMCSync"},
	{ObexObjectPushServiceClassID,		"Obex Object Push"},
	{ObexFileTransferServiceClassID,	"Obex File Transfer"},
	{IrMCSyncCommandServiceClassID,		"IrMCSync Command"},
	{HeadsetServiceClassID,			"Headset"},
	{CordlessTelephonyServiceClassID,	"Cordless Telephony"},
	{IntercomServiceClassID,		"Intercom"},
	{FaxServiceClassID,			"Fax"},
	{HeadsetAudioGatewayServiceClassID,	"Headset Audio Gateway"},
	{PnPInformationServiceClassID,		"PnP Information"},
	{GenericNetworkingServiceClassID,	"Generic Networking"},
	{GenericFileTransferServiceClassID,	"Generic File Transfer"},
	{GenericAudioServiceClassID,		"Generic Audio"},
	{GenericTelephonyServiceClassID,	"Generic Telephony"},
	{HandsFreeServiceClassID,		"Handsfree"},
	{AGHandsFreeServiceClassID,		"Handsfree Audio Gateway"},
};

struct tupla ProfileDescriptorMnemonic[] = {
	{SerialPortProfileID,			"Serial Port"},
	{LANAccessUsingPPPProfileID,		"LAN Access Using PPP"},
	{DialupNetworkingProfileID,		"Dialup Networking"},
	{IrMCSyncProfileID,			"IrMCSync"},
	{OBEXObjectPushProfileID,		"Obex Object Push"},
	{OBEXFileTransferProfileID,		"Obex File Transfer"},
	{IrMCSyncCommandProfileID,		"IrMCSync Command"},
	{HeadsetProfileID,			"Headset"},
	{CordlessTelephonyProfileID,		"Cordless Telephony"},
	{IntercomProfileID,			"Intercom"},
	{FaxProfileID,				"Fax"},
	{HeadsetAudioGatewayProfileID,		"Headset Audio Gateway"},
	{HandsFreeServiceClassID,		"Handsfree"},
};


char *string_lookup(struct tupla *pt0, int index)
{
	struct tupla *pt;

	for (pt = pt0; pt->index; pt++) {
		if (pt->index == index) {
			return pt->str;
		}
	}
	return "";
}


/*
 * Prints into a string the Protocol UUID
 * coping a maximum of n characters.
 */

int messageUUID2strn(struct tupla *message, UUID *pUUID, char *str, int n)
{
	if (pUUID != NULL) {
		switch (pUUID->uuidType) {
			case bits_16:
				{
					char *str2 = string_lookup(message, pUUID->value.uuid16Bit);
					snprintf(str, n, str2);
				}
				break;
			case bits_32:
				{
					char *str2 = string_lookup(message, pUUID->value.uuid32Bit);
					snprintf(str, n, str2);
				}
				break;
			case bits_128:
				snprintf(str, n, "Error: This is uuid128");
				return -4;
			default:
				snprintf(str, n, "Enum type of UUID not set.");
				return -1;
		}
	} else {
		snprintf(str, n, "Null passed to print UUID");
		return -2;
	}
	return 0;
}

int ProtocolUUID2strn(UUID *pUUID, char *str, size_t n)
{
	return messageUUID2strn(ProtocolMnemonic, pUUID, str, n);
}


int ServiceClassUUID2strn(UUID *pUUID, char *str, size_t n)
{
	return messageUUID2strn(ServiceClassMnemonic, pUUID, str, n);
}


int ProfileDescriptorUUID2strn(UUID *pUUID, char *str, size_t n)
{
	return messageUUID2strn(ProfileDescriptorMnemonic, pUUID, str, n);
}


/*
 * Prints into a string the UUID
 * coping a maximum of n characters.
 */
int UUID2strn(UUID *pUUID, char *str, size_t n)
{
	if (pUUID != NULL) {
		switch (pUUID->uuidType) {
			case bits_16:
				snprintf(str, n, "%.4x", pUUID->value.uuid16Bit);
				break;
			case bits_32:
				snprintf(str, n, "%.8x", pUUID->value.uuid32Bit);
				break;
			case bits_128: {
					       unsigned int data0;
					       unsigned short data1;
					       unsigned short data2;
					       unsigned short data3;
					       unsigned int data4;
					       unsigned short data5;

					       memcpy(&data0, &pUUID->value.uuid128Bit.data[0], 4);
					       memcpy(&data1, &pUUID->value.uuid128Bit.data[4], 2);
					       memcpy(&data2, &pUUID->value.uuid128Bit.data[6], 2);
					       memcpy(&data3, &pUUID->value.uuid128Bit.data[8], 2);
					       memcpy(&data4, &pUUID->value.uuid128Bit.data[10], 4);
					       memcpy(&data5, &pUUID->value.uuid128Bit.data[14], 2);

					       snprintf(str, n, "%.8x-%.4x-%.4x-%.4x-%.8x%.4x",
							       ntohl(data0), ntohs(data1), ntohs(data2),
							       ntohs(data3), ntohl(data4), ntohs(data5));
				       }
				       break;
			default:
				       snprintf(str, n, "Enum type of UUID not set.");
				       return -1;
		}
	} else {
		snprintf(str, n, "Null passed to print UUID");
		return -2;
	}

	return 0;	// OK
}

int UUID2value(UUID *pUUID)
{
	if (pUUID != NULL) {
		if (pUUID->uuidType == bits_16)
			return pUUID->value.uuid16Bit;
		else
			return pUUID->value.uuid32Bit;
	}
	return 0;
}


/*
 ** Function prints the UUID in hex as per defined syntax -
 **
 ** 4bytes-2bytes-2bytes-2bytes-6bytes
 **
 ** There is some ugly code, including hardcoding, but
 ** that is just the way it is converting 16 and 32 bit
 ** UUIDs to 128 bit as defined in the SDP doc
 **
 */
void printUUID(UUID *pUUID)
{
	char	uuid[80];
	
	UUID2strn(pUUID, uuid, sizeof(uuid));
	switch (pUUID->uuidType) {
		case bits_16:
			BTDEBUG("  UUID16Bit : 0x%s", uuid);
			break;
		case bits_32:
			BTDEBUG("  UUID32Bit : 0x%s", uuid);
			break;
		case bits_128:
			BTDEBUG("  UUID128Bit : 0x%s", uuid);
			break;
		default:
			BTDEBUG("%s", uuid);			
	}
}


/*
 ** The code in this function is executed only once per
 ** thread. We compute the actual bit value of the Bluetooth
 ** base UUID which is a string defined in sdp.h 
 ** and is assumed to be of the standard
 ** form with "-" separators.
 **
 ** The algorithm however converts the string to 4 unsigned longs
 ** using the strtoul() and assigns the values in sequence to
 ** the 128bit value
 */
UUID128Bit *makeBluetoothBaseUUID(void)
{
	char baseStr[128];
	int delim = '-';
	unsigned long dataLongValue;
	char *delimPtr;
	char *dataPtr;
	char temp[10];
	int toBeCopied;
	int status = 0;       
	char *data;

	if (pBluetoothBaseUUID128Bit == NULL) {
		strcpy(baseStr, BASE_UUID);
		pBluetoothBaseUUID128Bit = 
			(UUID128Bit *)malloc(sizeof(UUID128Bit));
		data = pBluetoothBaseUUID128Bit->data;
		memset(data, '\0', sizeof(UUID128Bit));

		memset(temp, '\0', 10);
		dataPtr = baseStr; delimPtr = NULL;
		delimPtr = strchr(dataPtr, delim);
		toBeCopied = delimPtr - dataPtr;
		if (toBeCopied == 8) {
			strncpy(temp, dataPtr, toBeCopied);
			dataLongValue = htonl(strtoul(temp, NULL, 16));
			memcpy(&data[0], &dataLongValue, 4);
		}
		else {
			BTDEBUG("To be copied(1) : %d\n", toBeCopied);
			status = -1;
		}

		/*
		 ** Get the next 4 bytes (note that there is a "-"
		 ** between them now
		 */
		if (status == 0) {
			memset(temp, '\0', 10); dataPtr = delimPtr + 1;
			delimPtr = strchr(dataPtr, delim);
			toBeCopied = delimPtr - dataPtr;
			if (toBeCopied == 4) {
				strncpy(temp, dataPtr, toBeCopied);

				dataPtr = delimPtr + 1;
				delimPtr = strchr(dataPtr, delim);
				toBeCopied = delimPtr - dataPtr;
				if (toBeCopied == 4) {
					strncat(temp, dataPtr, toBeCopied);
					dataLongValue = htonl(strtoul(temp, NULL, 16));
					memcpy(&data[4], &dataLongValue, 4);
				}
				else {
					BTDEBUG("To be copied(3) : %d\n", toBeCopied);
					status = -1;
				}
			}
			else {
				BTDEBUG("To be copied(2) : %d\n", toBeCopied);
				status = -1;
			}
		}                            

		/*
		 ** Get the last 4 bytes (note that there are 6 bytes
		 ** after the last separator, which is truncated (2+4)
		 */
		if (status == 0) {
			memset(temp, '\0', 10); dataPtr = delimPtr + 1;
			dataPtr = delimPtr + 1;
			delimPtr = strchr(dataPtr, delim);
			toBeCopied = delimPtr - dataPtr;
			if (toBeCopied == 4) {
				strncpy(temp, dataPtr, toBeCopied);
				strncat(temp, (delimPtr+1), 4);
				dataLongValue = htonl(strtoul(temp, NULL, 16));
				memcpy(&data[8], &dataLongValue, 4);

				dataLongValue = htonl(strtoul(delimPtr+5, NULL, 16));
				memcpy(&data[12], &dataLongValue, 4);
			}
			else {
				BTDEBUG("To be copied(4) : %d\n", toBeCopied);
				status = -1;
			}
		}                             
	}

	if (status == 0)
		return pBluetoothBaseUUID128Bit;
	else
		return NULL;
}

void makeUUIDFrom16Bits(UUID *pUUID, UUID16Bit value16Bit)
{
	if (pUUID != NULL) {
		memset(pUUID, 0, sizeof(UUID));
		pUUID->uuidType = bits_16;
		pUUID->value.uuid16Bit = value16Bit;
	}
}

void makeUUIDFrom32Bits(UUID *pUUID, UUID32Bit value32Bit)
{
	if (pUUID != NULL) {
		memset(pUUID, 0, sizeof(UUID));
		pUUID->uuidType = bits_32;
		pUUID->value.uuid32Bit = value32Bit;
	}
}

void makeUUIDFrom128Bits(UUID *pUUID, UUID128Bit *pValue128Bit)
{
	if ((pUUID != NULL) && (pValue128Bit != NULL)) {
		memset(pUUID, 0, sizeof(UUID));
		pUUID->uuidType = bits_128;
		memcpy(&pUUID->value.uuid128Bit, pValue128Bit, sizeof(UUID128Bit));
	}
}

/*
 ** UUID comparison function
 **
 ** Returns 
 ** 0 if uuidValue1 == uuidValue2
 ** -1 if uuidValue1 NEQ uidValue2
 **    
 */
gint compareUUID(gconstpointer p1, gconstpointer p2)
{
	UUID *puuidValue1;
	UUID *puuidValue2;
	int status = -1;

	puuidValue1 = (UUID *)p1;
	puuidValue2 = (UUID *)p2;


#ifdef CONFIG_AFFIX_DEBUG
	printUUID(puuidValue1);
	printUUID(puuidValue2);
#endif


	if ((puuidValue1 != NULL) && (puuidValue2 != NULL)) {
		status = memcmp(&puuidValue1->value.uuid128Bit, 
				&puuidValue2->value.uuid128Bit, sizeof(UUID128Bit));
		BTDEBUG("UUID compare status : %d", status);
		//BTDUMP(&puuidValue1->value.uuid128Bit, sizeof(UUID128Bit));
		//BTDUMP(&puuidValue2->value.uuid128Bit, sizeof(UUID128Bit));
	}
	return status;
}

/*
 ** 128 to 16 bit and 32 to 16 bit UUID conversion functions
 ** yet to be implemented. Note that the input is in NBO in
 ** both 32 and 128 bit UUIDs and conversion is needed
 */
void convert16BitsTo128Bits(UUID *pUUID128Bit, UUID *pUUID16Bit)
{
	/*
	 ** We have a 16 bit value, which needs to be added to
	 ** bytes 3 and 4 (at indices 2 and 3) of the Bluetooth base
	 */
	unsigned short data1;
	/*
	 ** First allocate a 128bit UUID for ourselves and
	 ** set the Bluetooth base UUID in it
	 */
	UUID128Bit *pBTBase128Bit = makeBluetoothBaseUUID();
	pUUID128Bit->value.uuid128Bit = *pBTBase128Bit;
	pUUID128Bit->uuidType = bits_128;

	/*
	 ** Extract bytes 2 and 3 of 128bit BT base UUID
	 */
	memcpy(&data1, &pBTBase128Bit->data[2], 2);
	/*
	 ** Add the given UUID (16 bits)
	 */
	data1 += htons(pUUID16Bit->value.uuid16Bit);
	/*
	 ** Set bytes 2 and 3 of the 128 bit value
	 */
	memcpy(&pUUID128Bit->value.uuid128Bit.data[2], &data1, 2);
}

void convert32BitsTo128Bits(UUID *pUUID128Bit, UUID *pUUID32Bit)
{
	/*
	 ** We have a 32 bit value, which needs to be added to
	 ** bytes 1->4 (at indices 0 thru 3) of the Bluetooth base
	 */
	unsigned int data0;
	/*
	 ** First allocate a 128bit UUID for ourselves and
	 ** set the Bluetooth base UUID in it
	 */
	UUID128Bit *pBTBase128Bit = makeBluetoothBaseUUID();
	pUUID128Bit->value.uuid128Bit = *pBTBase128Bit;
	pUUID128Bit->uuidType = bits_128;


	/*
	 ** Extract first 4 bytes
	 */
	memcpy(&data0, &pBTBase128Bit->data[0], 4);
	/*
	 ** Add the given UUID (32bits)
	 */
	data0 += htonl(pUUID32Bit->value.uuid32Bit);
	/*
	 ** Set the 4 bytes of the 128 bit value
	 */
	memcpy(&pUUID128Bit->value.uuid128Bit.data[0], &data0, 4);
}                                   

UUID *convertTo128Bits(UUID *pUUID)
{
	UUID *pUUID128Bit = NULL;

	pUUID128Bit = (UUID *)malloc(sizeof(UUID));
	memset(pUUID128Bit, 0, sizeof(UUID));
	switch (pUUID->uuidType) {
		case bits_128 :
			*pUUID128Bit = *pUUID;
			break;
		case bits_32 :
			convert32BitsTo128Bits(pUUID128Bit, pUUID);
			break;
		case bits_16 :
			convert16BitsTo128Bits(pUUID128Bit, pUUID);
			break;
	}

	return pUUID128Bit;
}

int generateUUIDPduForm(UUID *pUUID, char *buffer)
{
	int length = 0;
	char *pdata;

	pdata = buffer;

	if (pUUID->uuidType == bits_16) {
		__put_u8(pdata, UUID_16);
		pdata += sizeof(DataTypeDescriptor);
		length += sizeof(DataTypeDescriptor);
		__put_u16(pdata, htons(pUUID->value.uuid16Bit));
		length += sizeof(UUID16Bit);
	}
	else
		if (pUUID->uuidType == bits_32) {
			__put_u8(pdata, UUID_32);
			pdata += sizeof(DataTypeDescriptor);
			__put_u32(pdata, htonl(pUUID->value.uuid32Bit));
			length += sizeof(UUID32Bit);
		}
		else
			if (pUUID->uuidType == bits_128) {
				__put_u8(pdata, UUID_128);
				pdata += sizeof(DataTypeDescriptor);
				memcpy(pdata, &pUUID->value.uuid128Bit, sizeof(UUID128Bit));
				length += sizeof(UUID128Bit);
			}
			else
				BTERROR("No UUID type set in genUUIDPDUFrom\n");

	return length;
}

int extractUUID(char *pduBuffer, UUID *pUUID, int *bytesScanned)
{
	int status = 0;
	DataTypeDescriptor uuidType;
	char *pdata;

	pdata = pduBuffer; 
	uuidType = __get_u8(pdata);
	pdata += sizeof(DataTypeDescriptor);
	*bytesScanned += sizeof(DataTypeDescriptor);

	if ((uuidType != UUID_16) &&
			(uuidType != UUID_32) &&
			(uuidType != UUID_128)) {
		status = -1;
		BTERROR("Unknown data type : %d expecting a svc UUID\n", uuidType);
	}
	else {
		if (uuidType == UUID_16) {
			makeUUIDFrom16Bits(pUUID, ntohs(__get_u16(pdata)));
			*bytesScanned += sizeof(UUID16Bit);
			pdata += sizeof(UUID16Bit);
		}
		else
			if (uuidType == UUID_32) {
				makeUUIDFrom32Bits(pUUID, ntohl(__get_u32(pdata)));
				*bytesScanned += sizeof(UUID32Bit);
				pdata += sizeof(UUID32Bit);
			}
			else {
				makeUUIDFrom128Bits(pUUID, (UUID128Bit *)pdata);
				*bytesScanned += sizeof(UUID128Bit);
				pdata += sizeof(UUID128Bit);
			}
	}

	return status;
}
