/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author:Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   */

/*
   $Id: worker.c,v 1.3 2002/08/30 12:33:30 kds Exp $

   Worker thread start and run() method

	Fixes:     Guruprasad Krishnamurthy <guruprasad.krishnamurthy@nokia.com>
*/

#include <affix/config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "utils.h"

#include "queue.h"
#include "request.h"

pthread_t *thrIdArray;
pthread_attr_t thrAttr;

/*
 ** Run method executed by the worker thread. Sleeps 
 ** until the request queue is populated. One of many worker(s)
 ** gets the newly added entry, and proceeds to process it
 */
void *run(void *arg)
{
	/*
	 **
	 */
	SDPRequest *ptr;

	ptr = NULL;
	for (;;) {
		ptr = removeFirstEntry();
		if (ptr != NULL) { 
			BTERROR("Non null entry : %d\n", (int)pthread_self());
			processRequest(ptr);
		}
	}

	return NULL;
}

/*
 ** Start "n" numbers of worker threads
 */
void startWorkers(int count)
{
	int i;

	pthread_attr_init(&thrAttr);
	//pthread_attr_setdetachstate(&thrAttr, PTHREAD_CREATE_DETACHED);
		
	thrIdArray = (pthread_t *)malloc(sizeof(pthread_t) * count);
	memset((void *)thrIdArray, 0, (sizeof(pthread_t) * count));

	BTDEBUG("Starting thread\n");
	for (i = 0; i < count; i++) {
		pthread_create(thrIdArray+i, &thrAttr, run, NULL);
		BTDEBUG("Thr Id : %ld\n", thrIdArray[i]);
	}
}


